from unittest import TestCase

from django.test import TestCase as DjangoTestCase
from django.urls import reverse

from wagtail.admin.search import SearchArea
from wagtail.admin.ui.sidebar import (
    LinkMenuItem,
    MainMenuModule,
    PageExplorerMenuItem,
    SearchModule,
    SubMenuItem,
)
from wagtail.telepath import JSContext
from wagtail.test.utils import WagtailTestUtils


class TestAdaptLinkMenuItem(TestCase):
    def test_adapt(self):
        packed = JSContext().pack(LinkMenuItem("link", "Link", "/link/"))

        self.assertEqual(
            packed,
            {
                "_type": "wagtail.sidebar.LinkMenuItem",
                "_args": [
                    {
                        "classnames": "",
                        "icon_name": "",
                        "label": "Link",
                        "name": "link",
                        "url": "/link/",
                        "attrs": None,
                    }
                ],
            },
        )

    def test_adapt_with_optional_parameters(self):
        packed = JSContext().pack(
            LinkMenuItem(
                "link",
                "Link",
                "/link/",
                icon_name="link-icon",
                classnames="some classes",
                attrs={"data-is-custom": "true"},
            )
        )

        self.assertEqual(
            packed,
            {
                "_type": "wagtail.sidebar.LinkMenuItem",
                "_args": [
                    {
                        "classnames": "some classes",
                        "icon_name": "link-icon",
                        "label": "Link",
                        "name": "link",
                        "url": "/link/",
                        "attrs": {"data-is-custom": "true"},
                    }
                ],
            },
        )


class TestAdaptSubMenuItem(TestCase):
    def test_adapt(self):
        packed = JSContext().pack(
            SubMenuItem(
                "sub-menu",
                "Sub menu",
                [
                    LinkMenuItem("link", "Link", "/link/", icon_name="link-icon"),
                ],
                footer_text="Footer text",
            )
        )

        self.assertEqual(
            packed,
            {
                "_type": "wagtail.sidebar.SubMenuItem",
                "_args": [
                    {
                        "name": "sub-menu",
                        "label": "Sub menu",
                        "icon_name": "",
                        "classnames": "",
                        "footer_text": "Footer text",
                    },
                    [
                        {
                            "_type": "wagtail.sidebar.LinkMenuItem",
                            "_args": [
                                {
                                    "name": "link",
                                    "label": "Link",
                                    "icon_name": "link-icon",
                                    "classnames": "",
                                    "url": "/link/",
                                    "attrs": None,
                                }
                            ],
                        }
                    ],
                ],
            },
        )

    def test_adapt_without_footer_text(self):
        packed = JSContext().pack(
            SubMenuItem(
                "sub-menu",
                "Sub menu",
                [
                    LinkMenuItem("link", "Link", "/link/", icon_name="link-icon"),
                ],
            )
        )

        self.assertEqual(
            packed,
            {
                "_type": "wagtail.sidebar.SubMenuItem",
                "_args": [
                    {
                        "name": "sub-menu",
                        "label": "Sub menu",
                        "icon_name": "",
                        "classnames": "",
                        "footer_text": "",
                    },
                    [
                        {
                            "_type": "wagtail.sidebar.LinkMenuItem",
                            "_args": [
                                {
                                    "name": "link",
                                    "label": "Link",
                                    "icon_name": "link-icon",
                                    "classnames": "",
                                    "url": "/link/",
                                    "attrs": None,
                                }
                            ],
                        }
                    ],
                ],
            },
        )


class TestAdaptPageExplorerMenuItem(TestCase):
    def test_adapt(self):
        packed = JSContext().pack(PageExplorerMenuItem("pages", "Pages", "/pages/", 1))

        self.assertEqual(
            packed,
            {
                "_type": "wagtail.sidebar.PageExplorerMenuItem",
                "_args": [
                    {
                        "attrs": None,
                        "classnames": "",
                        "icon_name": "",
                        "label": "Pages",
                        "name": "pages",
                        "url": "/pages/",
                    },
                    1,
                ],
            },
        )


class TestAdaptSearchModule(TestCase):
    def test_adapt(self):
        packed = JSContext().pack(SearchModule(SearchArea("Search", "/search/")))

        self.assertEqual(
            packed, {"_type": "wagtail.sidebar.SearchModule", "_args": ["/search/"]}
        )


class TestAdaptMainMenuModule(DjangoTestCase, WagtailTestUtils):
    def test_adapt(self):
        main_menu = [
            LinkMenuItem("pages", "Pages", "/pages/"),
        ]
        account_menu = [
            LinkMenuItem(
                "account", "Account", reverse("wagtailadmin_account"), icon_name="user"
            ),
            LinkMenuItem(
                "logout", "Logout", reverse("wagtailadmin_logout"), icon_name="logout"
            ),
        ]
        user = self.create_user(username="admin")

        packed = JSContext().pack(MainMenuModule(main_menu, account_menu, user))

        self.assertEqual(
            packed,
            {
                "_type": "wagtail.sidebar.MainMenuModule",
                "_args": [
                    [
                        {
                            "_type": "wagtail.sidebar.LinkMenuItem",
                            "_args": [
                                {
                                    "name": "pages",
                                    "label": "Pages",
                                    "icon_name": "",
                                    "classnames": "",
                                    "url": "/pages/",
                                    "attrs": None,
                                }
                            ],
                        }
                    ],
                    [
                        {
                            "_type": "wagtail.sidebar.LinkMenuItem",
                            "_args": [
                                {
                                    "name": "account",
                                    "label": "Account",
                                    "icon_name": "user",
                                    "classnames": "",
                                    "url": reverse("wagtailadmin_account"),
                                    "attrs": None,
                                }
                            ],
                        },
                        {
                            "_type": "wagtail.sidebar.LinkMenuItem",
                            "_args": [
                                {
                                    "name": "logout",
                                    "label": "Logout",
                                    "icon_name": "logout",
                                    "classnames": "",
                                    "url": reverse("wagtailadmin_logout"),
                                    "attrs": None,
                                }
                            ],
                        },
                    ],
                    {
                        "name": user.first_name or user.get_username(),
                        "avatarUrl": "//www.gravatar.com/avatar/e64c7d89f26bd1972efa854d13d7dd61?s=100&d=mm",
                    },
                ],
            },
        )



# Generated by Django 2.0.4 on 2018-05-04 15:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("tests", "0031_customdocument_file_size"),
    ]

    operations = [
        migrations.AddField(
            model_name="customimage",
            name="file_hash",
            field=models.CharField(blank=True, editable=False, max_length=40),
        ),
        migrations.AddField(
            model_name="customimagefilepath",
            name="file_hash",
            field=models.CharField(blank=True, editable=False, max_length=40),
        ),
    ]



from django.contrib.auth.models import Permission
from django.test import TestCase
from django.test.utils import override_settings
from django.urls import reverse

from wagtail.documents import get_document_model
from wagtail.test.utils import WagtailTestUtils

Document = get_document_model()


class TestDocumentBulkDeleteView(TestCase, WagtailTestUtils):
    def setUp(self):
        self.user = self.login()

        # Create documents to delete
        self.documents = [
            Document.objects.create(title=f"Test document - {i}") for i in range(1, 6)
        ]
        self.url = (
            reverse(
                "wagtail_bulk_action",
                args=(
                    "wagtaildocs",
                    "document",
                    "delete",
                ),
            )
            + "?"
        )
        for document in self.documents:
            self.url += f"id={document.id}&"

    def test_delete_with_limited_permissions(self):
        self.user.is_superuser = False
        self.user.user_permissions.add(
            Permission.objects.get(
                content_type__app_label="wagtailadmin", codename="access_admin"
            )
        )
        self.user.save()

        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)

        html = response.content.decode()
        self.assertInHTML(
            "<p>You don't have permission to delete these documents</p>", html
        )

        for document in self.documents:
            self.assertInHTML(
                "<li>{document_title}</li>".format(document_title=document.title), html
            )

        response = self.client.post(self.url)
        # User should be redirected back to the index
        self.assertEqual(response.status_code, 302)

        # Documents should not be deleted
        for document in self.documents:
            self.assertTrue(Document.objects.filter(id=document.id).exists())

    def test_simple(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "wagtaildocs/bulk_actions/confirm_bulk_delete.html"
        )

    def test_delete(self):
        # Make post request
        response = self.client.post(self.url)

        # User should be redirected back to the index
        self.assertEqual(response.status_code, 302)

        # Documents should be deleted
        for document in self.documents:
            self.assertFalse(Document.objects.filter(id=document.id).exists())

    @override_settings(WAGTAIL_USAGE_COUNT_ENABLED=True)
    def test_usage_link(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "wagtaildocs/bulk_actions/confirm_bulk_delete.html"
        )
        for document in self.documents:
            self.assertContains(response, document.usage_url)
        # usage count should be printed for each document
        self.assertContains(response, "Used 0 times", count=5)



# Generated by Django 2.1.13 on 2019-10-17 13:25

from django.db import migrations


def assign_unlock_grouppagepermission(apps, schema_editor):
    # Assign Unlock permission to all groups that currently have the lock permission
    GroupPagePermission = apps.get_model("wagtailcore.GroupPagePermission")

    for lock_permission in GroupPagePermission.objects.filter(permission_type="lock"):
        GroupPagePermission.objects.create(
            group=lock_permission.group,
            page=lock_permission.page,
            permission_type="unlock",
        )


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailcore", "0044_add_unlock_grouppagepermission"),
    ]

    operations = [
        migrations.RunPython(
            assign_unlock_grouppagepermission, migrations.RunPython.noop
        )
    ]



import datetime

import django_filters
from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.exceptions import PermissionDenied
from django.utils.translation import gettext_lazy as _

from wagtail.admin.filters import DateRangePickerWidget, WagtailFilterSet
from wagtail.models import Page, UserPagePermissionsProxy

from .base import PageReportView


def get_users_for_filter():
    User = get_user_model()
    return User.objects.filter(locked_pages__isnull=False).order_by(User.USERNAME_FIELD)


class LockedPagesReportFilterSet(WagtailFilterSet):
    locked_at = django_filters.DateFromToRangeFilter(widget=DateRangePickerWidget)
    locked_by = django_filters.ModelChoiceFilter(
        field_name="locked_by", queryset=lambda request: get_users_for_filter()
    )

    class Meta:
        model = Page
        fields = ["locked_by", "locked_at", "live"]


class LockedPagesView(PageReportView):
    template_name = "wagtailadmin/reports/locked_pages.html"
    title = _("Locked pages")
    header_icon = "lock"
    list_export = PageReportView.list_export + [
        "locked_at",
        "locked_by",
    ]
    filterset_class = LockedPagesReportFilterSet

    def get_filename(self):
        return "locked-pages-report-{}".format(
            datetime.datetime.today().strftime("%Y-%m-%d")
        )

    def get_queryset(self):
        pages = (
            (
                UserPagePermissionsProxy(self.request.user).editable_pages()
                | Page.objects.filter(locked_by=self.request.user)
            )
            .filter(locked=True)
            .specific(defer=True)
        )

        if getattr(settings, "WAGTAIL_I18N_ENABLED", False):
            pages = pages.select_related("locale")

        self.queryset = pages
        return super().get_queryset()

    def dispatch(self, request, *args, **kwargs):
        if not UserPagePermissionsProxy(request.user).can_remove_locks():
            raise PermissionDenied
        return super().dispatch(request, *args, **kwargs)



from io import BytesIO

import PIL.Image
from django.core.files.images import ImageFile

from wagtail.images import get_image_model

Image = get_image_model()


def get_test_image_file(filename="test.png", colour="white", size=(640, 480)):
    f = BytesIO()
    image = PIL.Image.new("RGBA", size, colour)
    image.save(f, "PNG")
    return ImageFile(f, name=filename)


def get_test_image_file_jpeg(filename="test.jpg", colour="white", size=(640, 480)):
    f = BytesIO()
    image = PIL.Image.new("RGB", size, colour)
    image.save(f, "JPEG")
    return ImageFile(f, name=filename)


def get_test_image_file_webp(filename="test.webp", colour="white", size=(640, 480)):
    f = BytesIO()
    image = PIL.Image.new("RGB", size, colour)
    image.save(f, "WEBP")
    return ImageFile(f, name=filename)



from warnings import warn

from wagtail.admin.panels import FieldPanel
from wagtail.utils.deprecation import RemovedInWagtail50Warning


class SnippetChooserPanel(FieldPanel):
    def __init__(self, *args, **kwargs):
        warn(
            "wagtail.snippets.edit_handlers.SnippetChooserPanel is obsolete and should be replaced by wagtail.admin.panels.FieldPanel",
            category=RemovedInWagtail50Warning,
            stacklevel=2,
        )
        super().__init__(*args, **kwargs)



from django.conf import settings

from wagtail.models import Page


def get_pages_with_direct_explore_permission(user):
    # Get all pages that the user has direct add/edit/publish/lock permission on
    if user.is_superuser:
        # superuser has implicit permission on the root node
        return Page.objects.filter(depth=1)
    else:
        return Page.objects.filter(
            group_permissions__group__in=user.groups.all(),
            group_permissions__permission_type__in=["add", "edit", "publish", "lock"],
        )


def get_explorable_root_page(user):
    # Get the highest common explorable ancestor for the given user. If the user
    # has no permissions over any pages, this method will return None.
    pages = get_pages_with_direct_explore_permission(user)
    try:
        root_page = pages.first_common_ancestor(include_self=True, strict=True)
    except Page.DoesNotExist:
        root_page = None

    return root_page


def get_site_for_user(user):
    root_page = get_explorable_root_page(user)
    if root_page:
        root_site = root_page.get_site()
    else:
        root_site = None
    real_site_name = None
    if root_site:
        real_site_name = (
            root_site.site_name if root_site.site_name else root_site.hostname
        )
    return {
        "root_page": root_page,
        "root_site": root_site,
        "site_name": real_site_name if real_site_name else settings.WAGTAIL_SITE_NAME,
    }



from .add_tags import AddTagsBulkAction
from .add_to_collection import AddToCollectionBulkAction
from .delete import DeleteBulkAction

__all__ = ["AddToCollectionBulkAction", "AddTagsBulkAction", "DeleteBulkAction"]



from __future__ import absolute_import, unicode_literals

from django.conf.urls.i18n import i18n_patterns
from django.urls import include, path

from wagtail import urls as wagtail_urls

urlpatterns = i18n_patterns(path("", include(wagtail_urls)))



import json
from datetime import timedelta
from unittest import mock

from django.conf import settings
from django.template import Context, Template
from django.test import TestCase
from django.test.utils import override_settings
from django.utils import timezone
from django.utils.html import format_html
from freezegun import freeze_time

from wagtail.admin.staticfiles import versioned_static
from wagtail.admin.templatetags.wagtailadmin_tags import (
    avatar_url,
    i18n_enabled,
    locale_label_from_id,
)
from wagtail.admin.templatetags.wagtailadmin_tags import locales as locales_tag
from wagtail.admin.templatetags.wagtailadmin_tags import (
    notification_static,
    timesince_last_update,
    timesince_simple,
)
from wagtail.admin.ui.components import Component
from wagtail.images.tests.utils import get_test_image_file
from wagtail.models import Locale
from wagtail.test.utils import WagtailTestUtils
from wagtail.users.models import UserProfile


class TestAvatarTemplateTag(TestCase, WagtailTestUtils):
    def setUp(self):
        # Create a user
        self.test_user = self.create_user(
            username="testuser",
            email="testuser@email.com",
            password="password",
        )

    def test_use_gravatar_by_default(self):
        url = avatar_url(self.test_user)
        self.assertIn("www.gravatar.com", url)

    def test_skip_gravatar_if_no_email(self):
        self.test_user.email = ""
        url = avatar_url(self.test_user)
        self.assertIn("default-user-avatar", url)

    @override_settings(WAGTAIL_GRAVATAR_PROVIDER_URL="https://robohash.org")
    def test_custom_gravatar_provider(self):
        url = avatar_url(self.test_user)
        self.assertIn("robohash.org", url)

    @override_settings(WAGTAIL_GRAVATAR_PROVIDER_URL=None)
    def test_disable_gravatar(self):
        url = avatar_url(self.test_user)
        self.assertIn("default-user-avatar", url)

    def test_uploaded_avatar(self):
        user_profile = UserProfile.get_for_user(self.test_user)
        user_profile.avatar = get_test_image_file(filename="custom-avatar.png")
        user_profile.save()

        url = avatar_url(self.test_user)
        self.assertIn("custom-avatar", url)


class TestNotificationStaticTemplateTag(TestCase):
    @override_settings(STATIC_URL="/static/")
    def test_local_notification_static(self):
        url = notification_static("wagtailadmin/images/email-header.jpg")
        self.assertEqual(
            "{}/static/wagtailadmin/images/email-header.jpg".format(
                settings.WAGTAILADMIN_BASE_URL
            ),
            url,
        )

    @override_settings(
        STATIC_URL="/static/", WAGTAILADMIN_BASE_URL="http://localhost:8000"
    )
    def test_local_notification_static_baseurl(self):
        url = notification_static("wagtailadmin/images/email-header.jpg")
        self.assertEqual(
            "http://localhost:8000/static/wagtailadmin/images/email-header.jpg", url
        )

    @override_settings(
        STATIC_URL="https://s3.amazonaws.com/somebucket/static/",
        WAGTAILADMIN_BASE_URL="http://localhost:8000",
    )
    def test_remote_notification_static(self):
        url = notification_static("wagtailadmin/images/email-header.jpg")
        self.assertEqual(
            "https://s3.amazonaws.com/somebucket/static/wagtailadmin/images/email-header.jpg",
            url,
        )


class TestVersionedStatic(TestCase):
    def test_versioned_static(self):
        result = versioned_static("wagtailadmin/js/core.js")
        self.assertRegex(result, r"^/static/wagtailadmin/js/core.js\?v=(\w+)$")

    @mock.patch("wagtail.admin.staticfiles.static")
    def test_versioned_static_version_string(self, mock_static):
        mock_static.return_value = "/static/wagtailadmin/js/core.js?v=123"
        result = versioned_static("wagtailadmin/js/core.js")
        self.assertEqual(result, "/static/wagtailadmin/js/core.js?v=123")
        mock_static.assert_called_once_with("wagtailadmin/js/core.js")

    def test_versioned_static_absolute_path(self):
        result = versioned_static("/static/wagtailadmin/js/core.js")
        self.assertEqual(result, "/static/wagtailadmin/js/core.js")

    def test_versioned_static_url(self):
        result = versioned_static("http://example.org/static/wagtailadmin/js/core.js")
        self.assertEqual(result, "http://example.org/static/wagtailadmin/js/core.js")


@freeze_time("2020-07-01 12:00:00")
class TestTimesinceTags(TestCase):
    def test_timesince_simple(self):
        now = timezone.now()
        ts = timesince_simple(now)
        self.assertEqual(ts, "Just now")

        ts = timesince_simple(now - timedelta(hours=1, minutes=10))
        self.assertEqual(ts, "1\xa0hour ago")

        ts = timesince_simple(now - timedelta(weeks=2, hours=1, minutes=10))
        self.assertEqual(ts, "2\xa0weeks ago")

    def test_timesince_last_update_today_shows_time(self):
        dt = timezone.now() - timedelta(hours=1)
        formatted_time = dt.astimezone(timezone.get_current_timezone()).strftime(
            "%H:%M"
        )

        timesince = timesince_last_update(dt)
        self.assertEqual(timesince, formatted_time)

        # Check prefix output
        timesince = timesince_last_update(dt, time_prefix="my prefix")
        self.assertEqual(timesince, "my prefix {}".format(formatted_time))

        # Check user output
        timesince = timesince_last_update(dt, user_display_name="Gary")
        self.assertEqual(timesince, "{} by Gary".format(formatted_time))

        # Check user and prefix output
        timesince = timesince_last_update(
            dt, time_prefix="my prefix", user_display_name="Gary"
        )
        self.assertEqual(timesince, "my prefix {} by Gary".format(formatted_time))

    def test_timesince_last_update_before_today_shows_timeago(self):
        dt = timezone.now() - timedelta(weeks=1, days=2)

        timesince = timesince_last_update(dt, use_shorthand=False)
        self.assertEqual(timesince, "1\xa0week, 2\xa0days ago")

        timesince = timesince_last_update(dt)
        self.assertEqual(timesince, "1\xa0week ago")


class TestComponentTag(TestCase):
    def test_passing_context_to_component(self):
        class MyComponent(Component):
            def render_html(self, parent_context):
                return format_html(
                    "<h1>{} was here</h1>", parent_context.get("first_name")
                )

        template = Template(
            "{% load wagtailadmin_tags %}{% with first_name='Kilroy' %}{% component my_component %}{% endwith %}"
        )
        html = template.render(Context({"my_component": MyComponent()}))
        self.assertEqual(html, "<h1>Kilroy was here</h1>")

    def test_component_escapes_unsafe_strings(self):
        class MyComponent(Component):
            def render_html(self, parent_context):
                return "Look, I'm running with scissors! 8< 8< 8<"

        template = Template(
            "{% load wagtailadmin_tags %}<h1>{% component my_component %}</h1>"
        )
        html = template.render(Context({"my_component": MyComponent()}))
        self.assertEqual(
            html, "<h1>Look, I&#x27;m running with scissors! 8&lt; 8&lt; 8&lt;</h1>"
        )

    def test_error_on_rendering_non_component(self):
        template = Template(
            "{% load wagtailadmin_tags %}<h1>{% component my_component %}</h1>"
        )

        with self.assertRaises(ValueError) as cm:
            template.render(Context({"my_component": "hello"}))
        self.assertEqual(str(cm.exception), "Cannot render 'hello' as a component")


@override_settings(
    WAGTAIL_CONTENT_LANGUAGES=[
        ("en", "English"),
        ("fr", "French"),
        ("ro", "Romanian"),
        ("ru", "Russian"),
    ]
)
class TestInternationalisationTags(TestCase):
    def setUp(self):
        self.locale_ids = []
        for language_code in ["en", "fr", "ro", "ru"]:
            locale, _ = Locale.objects.get_or_create(language_code=language_code)
            self.locale_ids.append(locale.pk)

    def test_i18n_enabled(self):
        with override_settings(WAGTAIL_I18N_ENABLED=False):
            self.assertFalse(i18n_enabled())

        with override_settings(WAGTAIL_I18N_ENABLED=True):
            self.assertTrue(i18n_enabled())

    def test_locales(self):
        locales_output = locales_tag()
        self.assertIsInstance(locales_output, str)
        self.assertEqual(
            json.loads(locales_output),
            [
                {"code": "en", "display_name": "English"},
                {"code": "fr", "display_name": "French"},
                {"code": "ro", "display_name": "Romanian"},
                {"code": "ru", "display_name": "Russian"},
            ],
        )

    def test_locale_label_from_id(self):
        with self.assertNumQueries(1):
            self.assertEqual(locale_label_from_id(self.locale_ids[0]), "English")

        with self.assertNumQueries(0):
            self.assertEqual(locale_label_from_id(self.locale_ids[1]), "French")

        # check with an invalid id
        with self.assertNumQueries(0):
            self.assertIsNone(locale_label_from_id(self.locale_ids[-1] + 100), None)



from django.urls import path

from . import views

app_name = "wagtailsettings"
urlpatterns = [
    path("<slug:app_name>/<slug:model_name>/", views.edit_current_site, name="edit"),
    path("<slug:app_name>/<slug:model_name>/<int:site_pk>/", views.edit, name="edit"),
]



from django import template
from django.shortcuts import resolve_url
from django.template.defaulttags import token_kwargs
from django.template.loader import render_to_string
from django.utils.encoding import force_str
from django.utils.html import conditional_escape

from wagtail import VERSION, __version__
from wagtail.models import Page, Site
from wagtail.rich_text import RichText, expand_db_html
from wagtail.utils.version import get_main_version

register = template.Library()


@register.simple_tag(takes_context=True)
def pageurl(context, page, fallback=None):
    """
    Outputs a page's URL as relative (/foo/bar/) if it's within the same site as the
    current page, or absolute (http://example.com/foo/bar/) if not.
    If kwargs contains a fallback view name and page is None, the fallback view url will be returned.
    """
    if page is None and fallback:
        return resolve_url(fallback)

    if not hasattr(page, "relative_url"):
        raise ValueError("pageurl tag expected a Page object, got %r" % page)

    try:
        site = Site.find_for_request(context["request"])
        current_site = site
    except KeyError:
        # request not available in the current context; fall back on page.url
        return page.url

    if current_site is None:
        # request does not correspond to a recognised site; fall back on page.url
        return page.url

    # Pass page.relative_url the request object, which may contain a cached copy of
    # Site.get_site_root_paths()
    # This avoids page.relative_url having to make a database/cache fetch for this list
    # each time it's called.
    return page.relative_url(current_site, request=context.get("request"))


@register.simple_tag(takes_context=True)
def slugurl(context, slug):
    """
    Returns the URL for the page that has the given slug.

    First tries to find a page on the current site. If that fails or a request
    is not available in the context, then returns the URL for the first page
    that matches the slug on any site.
    """

    page = None
    try:
        site = Site.find_for_request(context["request"])
        current_site = site
    except KeyError:
        # No site object found - allow the fallback below to take place.
        pass
    else:
        if current_site is not None:
            page = Page.objects.in_site(current_site).filter(slug=slug).first()

    # If no page is found, fall back to searching the whole tree.
    if page is None:
        page = Page.objects.filter(slug=slug).first()

    if page:
        # call pageurl() instead of page.relative_url() here so we get the ``accepts_kwarg`` logic
        return pageurl(context, page)


@register.simple_tag
def wagtail_version():
    return __version__


@register.simple_tag
def wagtail_documentation_path():
    major, minor, patch, release, num = VERSION
    if release == "final":
        return "https://docs.wagtail.org/en/v%s" % __version__
    else:
        return "https://docs.wagtail.org/en/latest"


@register.simple_tag
def wagtail_release_notes_path():
    return "%s.html" % get_main_version(VERSION)


@register.filter
def richtext(value):
    if isinstance(value, RichText):
        # passing a RichText value through the |richtext filter should have no effect
        return value
    elif value is None:
        html = ""
    else:
        if isinstance(value, str):
            html = expand_db_html(value)
        else:
            raise TypeError(
                "'richtext' template filter received an invalid value; expected string, got {}.".format(
                    type(value)
                )
            )
    return render_to_string("wagtailcore/shared/richtext.html", {"html": html})


class IncludeBlockNode(template.Node):
    def __init__(self, block_var, extra_context, use_parent_context):
        self.block_var = block_var
        self.extra_context = extra_context
        self.use_parent_context = use_parent_context

    def render(self, context):
        try:
            value = self.block_var.resolve(context)
        except template.VariableDoesNotExist:
            return ""

        if hasattr(value, "render_as_block"):
            if self.use_parent_context:
                new_context = context.flatten()
            else:
                new_context = {}

            if self.extra_context:
                for var_name, var_value in self.extra_context.items():
                    new_context[var_name] = var_value.resolve(context)

            output = value.render_as_block(context=new_context)
        else:
            output = value

        if context.autoescape:
            return conditional_escape(output)
        else:
            return force_str(output)


@register.tag
def include_block(parser, token):
    """
    Render the passed item of StreamField content, passing the current template context
    if there's an identifiable way of doing so (i.e. if it has a `render_as_block` method).
    """
    tokens = token.split_contents()

    try:
        tag_name = tokens.pop(0)
        block_var_token = tokens.pop(0)
    except IndexError:
        raise template.TemplateSyntaxError(
            "%r tag requires at least one argument" % tag_name
        )

    block_var = parser.compile_filter(block_var_token)

    if tokens and tokens[0] == "with":
        tokens.pop(0)
        extra_context = token_kwargs(tokens, parser)
    else:
        extra_context = None

    use_parent_context = True
    if tokens and tokens[0] == "only":
        tokens.pop(0)
        use_parent_context = False

    if tokens:
        raise template.TemplateSyntaxError(
            "Unexpected argument to %r tag: %r" % (tag_name, tokens[0])
        )

    return IncludeBlockNode(block_var, extra_context, use_parent_context)


@register.simple_tag(takes_context=True)
def wagtail_site(context):
    """
    Returns the Site object for the given request
    """
    try:
        request = context["request"]
    except KeyError:
        return None

    return Site.find_for_request(request=request)



from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.core.paginator import Paginator
from django.http import Http404
from django.template.response import TemplateResponse
from django.views.decorators.vary import vary_on_headers

from wagtail.admin.auth import user_has_any_page_permission, user_passes_test
from wagtail.admin.forms.search import SearchForm
from wagtail.models import Page
from wagtail.search.query import MATCH_ALL
from wagtail.search.utils import parse_query_string


def page_filter_search(q, pages, all_pages=None, ordering=None):
    # Parse query
    filters, query = parse_query_string(q, operator="and", zero_terms=MATCH_ALL)

    # Live filter
    live_filter = filters.get("live") or filters.get("published")
    live_filter = live_filter and live_filter.lower()

    if live_filter in ["yes", "true"]:
        if all_pages is not None:
            all_pages = all_pages.filter(live=True)
        pages = pages.filter(live=True)
    elif live_filter in ["no", "false"]:
        if all_pages is not None:
            all_pages = all_pages.filter(live=False)
        pages = pages.filter(live=False)

    # Search
    if all_pages is not None:
        all_pages = all_pages.search(query, order_by_relevance=not ordering)
    pages = pages.search(query, order_by_relevance=not ordering)

    return pages, all_pages


@vary_on_headers("X-Requested-With")
@user_passes_test(user_has_any_page_permission)
def search(request):
    pages = all_pages = Page.objects.all().prefetch_related("content_type").specific()
    show_locale_labels = getattr(settings, "WAGTAIL_I18N_ENABLED", False)
    if show_locale_labels:
        pages = pages.select_related("locale")

    q = MATCH_ALL
    content_types = []
    ordering = None

    if "ordering" in request.GET:
        if request.GET["ordering"] in [
            "title",
            "-title",
            "latest_revision_created_at",
            "-latest_revision_created_at",
            "live",
            "-live",
        ]:
            ordering = request.GET["ordering"]

            if ordering == "title":
                pages = pages.order_by("title")
            elif ordering == "-title":
                pages = pages.order_by("-title")

            if ordering == "latest_revision_created_at":
                pages = pages.order_by("latest_revision_created_at")
            elif ordering == "-latest_revision_created_at":
                pages = pages.order_by("-latest_revision_created_at")

            if ordering == "live":
                pages = pages.order_by("live")
            elif ordering == "-live":
                pages = pages.order_by("-live")

    if "content_type" in request.GET:
        try:
            app_label, model_name = request.GET["content_type"].split(".")
        except ValueError:
            raise Http404

        try:
            selected_content_type = ContentType.objects.get_by_natural_key(
                app_label, model_name
            )
        except ContentType.DoesNotExist:
            raise Http404

        pages = pages.filter(content_type=selected_content_type)
    else:
        selected_content_type = None

    if "q" in request.GET:
        form = SearchForm(request.GET)
        if form.is_valid():
            q = form.cleaned_data["q"]

            # Parse query and filter
            pages, all_pages = page_filter_search(q, pages, all_pages, ordering)

            # Facets
            if pages.supports_facet:
                content_types = [
                    (ContentType.objects.get(id=content_type_id), count)
                    for content_type_id, count in all_pages.facet(
                        "content_type_id"
                    ).items()
                ]

    else:
        form = SearchForm()

    paginator = Paginator(pages, per_page=20)
    pages = paginator.get_page(request.GET.get("p"))

    if request.headers.get("x-requested-with") == "XMLHttpRequest":
        return TemplateResponse(
            request,
            "wagtailadmin/pages/search_results.html",
            {
                "pages": pages,
                "all_pages": all_pages,
                "query_string": q,
                "content_types": content_types,
                "selected_content_type": selected_content_type,
                "ordering": ordering,
                "show_locale_labels": show_locale_labels,
            },
        )
    else:
        return TemplateResponse(
            request,
            "wagtailadmin/pages/search.html",
            {
                "search_form": form,
                "pages": pages,
                "all_pages": all_pages,
                "query_string": q,
                "content_types": content_types,
                "selected_content_type": selected_content_type,
                "ordering": ordering,
                "show_locale_labels": show_locale_labels,
            },
        )



import django_filters
from django.utils.translation import gettext as _

from wagtail.admin.filters import WagtailFilterSet
from wagtail.admin.widgets import ButtonSelect
from wagtail.models import Site


class RedirectsReportFilterSet(WagtailFilterSet):
    is_permanent = django_filters.ChoiceFilter(
        label=_("Type"),
        method="filter_type",
        choices=(
            (True, _("Permanent")),
            (False, _("Temporary")),
        ),
        empty_label=_("All"),
        widget=ButtonSelect,
    )

    site = django_filters.ModelChoiceFilter(
        field_name="site", queryset=Site.objects.all()
    )

    def filter_type(self, queryset, name, value):
        if value and self.request and self.request.user:
            queryset = queryset.filter(is_permanent=value)
        return queryset



"""
Tests for the search box in the admin side menu, and the custom search hooks.
"""
from django.contrib.auth.models import Permission
from django.template import Context, Template
from django.test import RequestFactory, TestCase
from django.urls import reverse

from wagtail.admin.auth import user_has_any_page_permission
from wagtail.test.utils import WagtailTestUtils


class BaseSearchAreaTestCase(WagtailTestUtils, TestCase):
    rf = RequestFactory()

    def search_other(self, current_url="/admin/", data=None):
        request = self.rf.get(current_url, data=data)
        request.user = self.user
        template = Template("{% load wagtailadmin_tags %}{% search_other %}")
        return template.render(Context({"request": request}))


class TestSearchAreas(BaseSearchAreaTestCase):
    def setUp(self):
        super().setUp()
        self.user = self.login()

    def test_other_searches(self):
        search_url = reverse("wagtailadmin_pages:search")
        query = "Hello"
        base_css = "search--custom-class"
        icon = '<svg class="icon icon-custom filter-options__icon" aria-hidden="true"><use href="#icon-custom"></use></svg>'
        test_string = (
            '<a href="/customsearch/?q=%s" class="%s" is-custom="true">%sMy Search</a>'
        )
        # Testing the option link exists
        response = self.client.get(search_url, {"q": query})
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailadmin/pages/search.html")
        self.assertTemplateUsed(response, "wagtailadmin/shared/search_area.html")
        self.assertTemplateUsed(response, "wagtailadmin/shared/search_other.html")
        self.assertContains(response, test_string % (query, base_css, icon), html=True)

        # Testing is_shown
        response = self.client.get(search_url, {"q": query, "hide-option": "true"})
        self.assertNotContains(
            response, test_string % (query, base_css, icon), status_code=200, html=True
        )

        # Testing is_active
        response = self.client.get(search_url, {"q": query, "active-option": "true"})
        self.assertContains(
            response,
            test_string % (query, base_css + " nolink", icon),
            status_code=200,
            html=True,
        )

    def test_search_other(self):
        rendered = self.search_other()
        self.assertIn(reverse("wagtailadmin_pages:search"), rendered)
        self.assertIn("/customsearch/", rendered)

        self.assertIn("Pages", rendered)
        self.assertIn("My Search", rendered)


class TestSearchAreaNoPagePermissions(BaseSearchAreaTestCase):
    """
    Test the admin search when the user does not have permission to manage
    pages. The search bar should show the first available search area instead.
    """

    def setUp(self):
        self.user = self.login()
        self.assertFalse(user_has_any_page_permission(self.user))

    def create_test_user(self):
        user = super().create_test_user()
        user.is_superuser = False
        user.user_permissions.add(
            Permission.objects.get(
                content_type__app_label="wagtailadmin", codename="access_admin"
            )
        )
        user.save()
        return user

    def test_dashboard(self):
        """
        Check that the menu search area on the dashboard is not searching
        pages, as they are not allowed.
        """
        response = self.client.get("/admin/")
        # The menu search bar should go to /customsearch/, not /admin/pages/search/
        self.assertNotContains(response, reverse("wagtailadmin_pages:search"))
        self.assertContains(
            response,
            '{"_type": "wagtail.sidebar.SearchModule", "_args": ["/customsearch/"]}',
        )

    def test_search_other(self):
        """The pages search link should be hidden, custom search should be visible."""
        rendered = self.search_other()
        self.assertNotIn(reverse("wagtailadmin_pages:search"), rendered)
        self.assertIn("/customsearch/", rendered)

        self.assertNotIn("Pages", rendered)
        self.assertIn("My Search", rendered)



# Generated by Django 4.0.3 on 2022-03-28 09:18

from django.db import migrations, models
import wagtail.contrib.forms.models


class Migration(migrations.Migration):

    dependencies = [
        ("tests", "0063_jsonblockcountsstreammodel_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="formpage",
            name="from_address",
            field=models.EmailField(
                blank=True, max_length=255, verbose_name="from address"
            ),
        ),
        migrations.AlterField(
            model_name="formpage",
            name="to_address",
            field=models.CharField(
                blank=True,
                help_text="Optional - form submissions will be emailed to these addresses. Separate multiple addresses by comma.",
                max_length=255,
                validators=[wagtail.contrib.forms.models.validate_to_address],
                verbose_name="to address",
            ),
        ),
        migrations.AlterField(
            model_name="formpagewithcustomformbuilder",
            name="from_address",
            field=models.EmailField(
                blank=True, max_length=255, verbose_name="from address"
            ),
        ),
        migrations.AlterField(
            model_name="formpagewithcustomformbuilder",
            name="to_address",
            field=models.CharField(
                blank=True,
                help_text="Optional - form submissions will be emailed to these addresses. Separate multiple addresses by comma.",
                max_length=255,
                validators=[wagtail.contrib.forms.models.validate_to_address],
                verbose_name="to address",
            ),
        ),
        migrations.AlterField(
            model_name="formpagewithcustomsubmission",
            name="from_address",
            field=models.EmailField(
                blank=True, max_length=255, verbose_name="from address"
            ),
        ),
        migrations.AlterField(
            model_name="formpagewithcustomsubmission",
            name="to_address",
            field=models.CharField(
                blank=True,
                help_text="Optional - form submissions will be emailed to these addresses. Separate multiple addresses by comma.",
                max_length=255,
                validators=[wagtail.contrib.forms.models.validate_to_address],
                verbose_name="to address",
            ),
        ),
        migrations.AlterField(
            model_name="formpagewithcustomsubmissionlistview",
            name="from_address",
            field=models.EmailField(
                blank=True, max_length=255, verbose_name="from address"
            ),
        ),
        migrations.AlterField(
            model_name="formpagewithcustomsubmissionlistview",
            name="to_address",
            field=models.CharField(
                blank=True,
                help_text="Optional - form submissions will be emailed to these addresses. Separate multiple addresses by comma.",
                max_length=255,
                validators=[wagtail.contrib.forms.models.validate_to_address],
                verbose_name="to address",
            ),
        ),
        migrations.AlterField(
            model_name="formpagewithredirect",
            name="from_address",
            field=models.EmailField(
                blank=True, max_length=255, verbose_name="from address"
            ),
        ),
        migrations.AlterField(
            model_name="formpagewithredirect",
            name="to_address",
            field=models.CharField(
                blank=True,
                help_text="Optional - form submissions will be emailed to these addresses. Separate multiple addresses by comma.",
                max_length=255,
                validators=[wagtail.contrib.forms.models.validate_to_address],
                verbose_name="to address",
            ),
        ),
        migrations.AlterField(
            model_name="jadeformpage",
            name="from_address",
            field=models.EmailField(
                blank=True, max_length=255, verbose_name="from address"
            ),
        ),
        migrations.AlterField(
            model_name="jadeformpage",
            name="to_address",
            field=models.CharField(
                blank=True,
                help_text="Optional - form submissions will be emailed to these addresses. Separate multiple addresses by comma.",
                max_length=255,
                validators=[wagtail.contrib.forms.models.validate_to_address],
                verbose_name="to address",
            ),
        ),
    ]



import datetime
from functools import wraps

from django.test import TestCase
from django.urls import reverse
from django.utils import timezone
from freezegun import freeze_time

from wagtail.admin.views.pages.preview import PreviewOnEdit
from wagtail.models import Page
from wagtail.test.testapp.models import EventCategory, SimplePage, StreamPage
from wagtail.test.utils import WagtailTestUtils


class TestIssue2599(TestCase, WagtailTestUtils):
    """
    When previewing a page on creation, we need to assign it a path value consistent with its
    (future) position in the tree. The naive way of doing this is to give it an index number
    one more than numchild - however, index numbers are not reassigned on page deletion, so
    this can result in a path that collides with an existing page (which is invalid).
    """

    def test_issue_2599(self):
        homepage = Page.objects.get(id=2)

        child1 = Page(title="child1")
        homepage.add_child(instance=child1)
        child2 = Page(title="child2")
        homepage.add_child(instance=child2)

        child1.delete()

        self.login()
        post_data = {
            "title": "New page!",
            "content": "Some content",
            "slug": "hello-world",
            "action-submit": "Submit",
        }
        preview_url = reverse(
            "wagtailadmin_pages:preview_on_add",
            args=("tests", "simplepage", homepage.id),
        )
        response = self.client.post(preview_url, post_data)

        # Check the JSON response
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(response.content.decode(), {"is_valid": True})

        response = self.client.get(preview_url)

        # Check the HTML response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "tests/simple_page.html")
        self.assertContains(response, "New page!")

        # Check that the treebeard attributes were set correctly on the page object
        self.assertEqual(response.context["self"].depth, homepage.depth + 1)
        self.assertTrue(response.context["self"].path.startswith(homepage.path))
        self.assertEqual(response.context["self"].get_parent(), homepage)


def clear_edit_handler(page_cls):
    def decorator(fn):
        @wraps(fn)
        def decorated(*args, **kwargs):
            # Clear any old panel definitions generated
            page_cls.get_edit_handler.cache_clear()
            try:
                fn(*args, **kwargs)
            finally:
                # Clear the bad panel definition generated just now
                page_cls.get_edit_handler.cache_clear()

        return decorated

    return decorator


class TestPreview(TestCase, WagtailTestUtils):
    fixtures = ["test.json"]

    def setUp(self):
        self.meetings_category = EventCategory.objects.create(name="Meetings")
        self.parties_category = EventCategory.objects.create(name="Parties")
        self.holidays_category = EventCategory.objects.create(name="Holidays")

        self.home_page = Page.objects.get(url_path="/home/")
        self.event_page = Page.objects.get(url_path="/home/events/christmas/")

        self.user = self.login()

        self.post_data = {
            "title": "Beach party",
            "slug": "beach-party",
            "body": """{"entityMap": {},"blocks": [
                {"inlineStyleRanges": [], "text": "party on wayne", "depth": 0, "type": "unstyled", "key": "00000", "entityRanges": []}
            ]}""",
            "date_from": "2017-08-01",
            "audience": "public",
            "location": "the beach",
            "cost": "six squid",
            "carousel_items-TOTAL_FORMS": 0,
            "carousel_items-INITIAL_FORMS": 0,
            "carousel_items-MIN_NUM_FORMS": 0,
            "carousel_items-MAX_NUM_FORMS": 0,
            "speakers-TOTAL_FORMS": 0,
            "speakers-INITIAL_FORMS": 0,
            "speakers-MIN_NUM_FORMS": 0,
            "speakers-MAX_NUM_FORMS": 0,
            "related_links-TOTAL_FORMS": 0,
            "related_links-INITIAL_FORMS": 0,
            "related_links-MIN_NUM_FORMS": 0,
            "related_links-MAX_NUM_FORMS": 0,
            "head_counts-TOTAL_FORMS": 0,
            "head_counts-INITIAL_FORMS": 0,
            "head_counts-MIN_NUM_FORMS": 0,
            "head_counts-MAX_NUM_FORMS": 0,
            "categories": [self.parties_category.id, self.holidays_category.id],
            "comments-TOTAL_FORMS": 0,
            "comments-INITIAL_FORMS": 0,
            "comments-MIN_NUM_FORMS": 0,
            "comments-MAX_NUM_FORMS": 1000,
        }

    def test_preview_on_create_with_m2m_field(self):
        preview_url = reverse(
            "wagtailadmin_pages:preview_on_add",
            args=("tests", "eventpage", self.home_page.id),
        )
        response = self.client.post(preview_url, self.post_data)

        # Check the JSON response
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(response.content.decode(), {"is_valid": True})

        # Check the user can refresh the preview
        preview_session_key = "wagtail-preview-tests-eventpage-{}".format(
            self.home_page.id
        )
        self.assertIn(preview_session_key, self.client.session)

        response = self.client.get(preview_url)

        # Check the HTML response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "tests/event_page.html")
        self.assertContains(response, "Beach party")
        self.assertContains(response, "<li>Parties</li>")
        self.assertContains(response, "<li>Holidays</li>")

    def test_preview_on_edit_with_m2m_field(self):
        preview_url = reverse(
            "wagtailadmin_pages:preview_on_edit", args=(self.event_page.id,)
        )
        response = self.client.post(preview_url, self.post_data)

        # Check the JSON response
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(response.content.decode(), {"is_valid": True})

        # Check the user can refresh the preview
        preview_session_key = "wagtail-preview-{}".format(self.event_page.id)
        self.assertIn(preview_session_key, self.client.session)

        response = self.client.get(preview_url)

        # Check the HTML response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "tests/event_page.html")
        self.assertContains(response, "Beach party")
        self.assertContains(response, "<li>Parties</li>")
        self.assertContains(response, "<li>Holidays</li>")

    def test_preview_on_edit_expiry(self):
        initial_datetime = timezone.now()
        expiry_datetime = initial_datetime + datetime.timedelta(
            seconds=PreviewOnEdit.preview_expiration_timeout + 1
        )

        with freeze_time(initial_datetime) as frozen_datetime:
            preview_url = reverse(
                "wagtailadmin_pages:preview_on_edit", args=(self.event_page.id,)
            )
            response = self.client.post(preview_url, self.post_data)

            # Check the JSON response
            self.assertEqual(response.status_code, 200)

            response = self.client.get(preview_url)

            # Check the HTML response
            self.assertEqual(response.status_code, 200)

            frozen_datetime.move_to(expiry_datetime)

            preview_url = reverse(
                "wagtailadmin_pages:preview_on_edit", args=(self.home_page.id,)
            )
            response = self.client.post(preview_url, self.post_data)
            self.assertEqual(response.status_code, 200)
            response = self.client.get(preview_url)
            self.assertEqual(response.status_code, 200)


class TestDisablePreviewButton(TestCase, WagtailTestUtils):
    """
    Test that preview button can be disabled by setting preview_modes to an empty list
    """

    def setUp(self):
        # Find root page
        self.root_page = Page.objects.get(id=2)

        # Login
        self.user = self.login()

    def test_disable_preview_on_create(self):
        # preview button is available by default
        response = self.client.get(
            reverse(
                "wagtailadmin_pages:add",
                args=("tests", "simplepage", self.root_page.id),
            )
        )
        self.assertEqual(response.status_code, 200)

        preview_url = reverse(
            "wagtailadmin_pages:preview_on_add",
            args=("tests", "simplepage", self.root_page.id),
        )
        self.assertContains(response, '<li class="preview">')
        self.assertContains(response, 'data-action="%s"' % preview_url)

        # StreamPage has preview_modes = []
        response = self.client.get(
            reverse(
                "wagtailadmin_pages:add",
                args=("tests", "streampage", self.root_page.id),
            )
        )
        self.assertEqual(response.status_code, 200)

        preview_url = reverse(
            "wagtailadmin_pages:preview_on_add",
            args=("tests", "streampage", self.root_page.id),
        )
        self.assertNotContains(response, '<li class="preview">')
        self.assertNotContains(response, 'data-action="%s"' % preview_url)

    def test_disable_preview_on_edit(self):
        simple_page = SimplePage(title="simple page", content="hello")
        self.root_page.add_child(instance=simple_page)

        # preview button is available by default
        response = self.client.get(
            reverse("wagtailadmin_pages:edit", args=(simple_page.id,))
        )
        self.assertEqual(response.status_code, 200)

        preview_url = reverse(
            "wagtailadmin_pages:preview_on_edit", args=(simple_page.id,)
        )
        self.assertContains(response, '<li class="preview">')
        self.assertContains(response, 'data-action="%s"' % preview_url)

        stream_page = StreamPage(title="stream page", body=[("text", "hello")])
        self.root_page.add_child(instance=stream_page)

        # StreamPage has preview_modes = []
        response = self.client.get(
            reverse("wagtailadmin_pages:edit", args=(stream_page.id,))
        )
        self.assertEqual(response.status_code, 200)

        preview_url = reverse(
            "wagtailadmin_pages:preview_on_edit", args=(stream_page.id,)
        )
        self.assertNotContains(response, '<li class="preview">')
        self.assertNotContains(response, 'data-action="%s"' % preview_url)

    def test_disable_preview_on_revisions_list(self):
        simple_page = SimplePage(title="simple page", content="hello")
        self.root_page.add_child(instance=simple_page)
        simple_page.save_revision(log_action=True)

        # check preview shows up by default
        response = self.client.get(
            reverse("wagtailadmin_pages:history", args=(simple_page.id,))
        )
        preview_url = reverse(
            "wagtailadmin_pages:revisions_view",
            args=(simple_page.id, simple_page.get_latest_revision().id),
        )
        self.assertContains(response, "Preview")
        self.assertContains(response, preview_url)

        stream_page = StreamPage(title="stream page", body=[("text", "hello")])
        self.root_page.add_child(instance=stream_page)
        latest_revision = stream_page.save_revision(log_action=True)

        # StreamPage has preview_modes = []
        response = self.client.get(
            reverse("wagtailadmin_pages:history", args=(stream_page.id,))
        )
        preview_url = reverse(
            "wagtailadmin_pages:revisions_view",
            args=(stream_page.id, latest_revision.id),
        )
        self.assertNotContains(response, "Preview")
        self.assertNotContains(response, preview_url)

    def disable_preview_in_moderation_list(self):
        stream_page = StreamPage(title="stream page", body=[("text", "hello")])
        self.root_page.add_child(instance=stream_page)
        latest_revision = stream_page.save_revision(
            user=self.user, submitted_for_moderation=True
        )

        response = self.client.get(reverse("wagtailadmin_home"))
        preview_url = reverse(
            "wagtailadmin_pages:preview_for_moderation", args=(latest_revision.id,)
        )
        self.assertNotContains(response, '<li class="preview">')
        self.assertNotContains(response, 'data-action="%s"' % preview_url)



from urllib.parse import urlparse

from django.conf import settings
from django.utils.encoding import force_str

from wagtail.coreutils import resolve_model_string
from wagtail.models import Page, Site


class BadRequestError(Exception):
    pass


def get_base_url(request=None):
    base_url = getattr(settings, "WAGTAILAPI_BASE_URL", None)

    if base_url is None and request:
        site = Site.find_for_request(request)
        if site:
            base_url = site.root_url

    if base_url:
        # We only want the scheme and netloc
        base_url_parsed = urlparse(force_str(base_url))

        return base_url_parsed.scheme + "://" + base_url_parsed.netloc


def get_full_url(request, path):
    base_url = get_base_url(request) or ""
    return base_url + path


def get_object_detail_url(router, request, model, pk):
    url_path = router.get_object_detail_urlpath(model, pk)

    if url_path:
        return get_full_url(request, url_path)


def page_models_from_string(string):
    page_models = []

    for sub_string in string.split(","):
        page_model = resolve_model_string(sub_string)

        if not issubclass(page_model, Page):
            raise ValueError("Model is not a page")

        page_models.append(page_model)

    return tuple(page_models)


class FieldsParameterParseError(ValueError):
    pass


def parse_fields_parameter(fields_str):
    """
    Parses the ?fields= GET parameter. As this parameter is supposed to be used
    by developers, the syntax is quite tight (eg, not allowing any whitespace).
    Having a strict syntax allows us to extend the it at a later date with less
    chance of breaking anyone's code.

    This function takes a string and returns a list of tuples representing each
    top-level field. Each tuple contains three items:
     - The name of the field (string)
     - Whether the field has been negated (boolean)
     - A list of nested fields if there are any, None otherwise

    Some examples of how this function works:

    >>> parse_fields_parameter("foo")
    [
        ('foo', False, None),
    ]

    >>> parse_fields_parameter("foo,bar")
    [
        ('foo', False, None),
        ('bar', False, None),
    ]

    >>> parse_fields_parameter("-foo")
    [
        ('foo', True, None),
    ]

    >>> parse_fields_parameter("foo(bar,baz)")
    [
        ('foo', False, [
            ('bar', False, None),
            ('baz', False, None),
        ]),
    ]

    It raises a FieldsParameterParseError (subclass of ValueError) if it
    encounters a syntax error
    """

    def get_position(current_str):
        return len(fields_str) - len(current_str)

    def parse_field_identifier(fields_str):
        first_char = True
        negated = False
        ident = ""

        while fields_str:
            char = fields_str[0]

            if char in ["(", ")", ","]:
                if not ident:
                    raise FieldsParameterParseError(
                        "unexpected char '%s' at position %d"
                        % (char, get_position(fields_str))
                    )

                if ident in ["*", "_"] and char == "(":
                    # * and _ cannot have nested fields
                    raise FieldsParameterParseError(
                        "unexpected char '%s' at position %d"
                        % (char, get_position(fields_str))
                    )

                return ident, negated, fields_str

            elif char == "-":
                if not first_char:
                    raise FieldsParameterParseError(
                        "unexpected char '%s' at position %d"
                        % (char, get_position(fields_str))
                    )

                negated = True

            elif char in ["*", "_"]:
                if ident and char == "*":
                    raise FieldsParameterParseError(
                        "unexpected char '%s' at position %d"
                        % (char, get_position(fields_str))
                    )

                ident += char

            elif char.isalnum() or char == "_":
                if ident == "*":
                    # * can only be on its own
                    raise FieldsParameterParseError(
                        "unexpected char '%s' at position %d"
                        % (char, get_position(fields_str))
                    )

                ident += char

            elif char.isspace():
                raise FieldsParameterParseError(
                    "unexpected whitespace at position %d" % get_position(fields_str)
                )
            else:
                raise FieldsParameterParseError(
                    "unexpected char '%s' at position %d"
                    % (char, get_position(fields_str))
                )

            first_char = False
            fields_str = fields_str[1:]

        return ident, negated, fields_str

    def parse_fields(fields_str, expect_close_bracket=False):
        first_ident = None
        is_first = True
        fields = []

        while fields_str:
            sub_fields = None
            ident, negated, fields_str = parse_field_identifier(fields_str)

            # Some checks specific to '*' and '_'
            if ident in ["*", "_"]:
                if not is_first:
                    raise FieldsParameterParseError(
                        "'%s' must be in the first position" % ident
                    )

                if negated:
                    raise FieldsParameterParseError("'%s' cannot be negated" % ident)

            if fields_str and fields_str[0] == "(":
                if negated:
                    # Negated fields cannot contain subfields
                    raise FieldsParameterParseError(
                        "unexpected char '(' at position %d" % get_position(fields_str)
                    )

                sub_fields, fields_str = parse_fields(
                    fields_str[1:], expect_close_bracket=True
                )

            if is_first:
                first_ident = ident
            else:
                # Negated fields can't be used with '_'
                if first_ident == "_" and negated:
                    # _,foo is allowed but _,-foo is not
                    raise FieldsParameterParseError(
                        "negated fields with '_' doesn't make sense"
                    )

                # Additional fields without sub fields can't be used with '*'
                if first_ident == "*" and not negated and not sub_fields:
                    # *,foo(bar) and *,-foo are allowed but *,foo is not
                    raise FieldsParameterParseError(
                        "additional fields with '*' doesn't make sense"
                    )

            fields.append((ident, negated, sub_fields))

            if fields_str and fields_str[0] == ")":
                if not expect_close_bracket:
                    raise FieldsParameterParseError(
                        "unexpected char ')' at position %d" % get_position(fields_str)
                    )

                return fields, fields_str[1:]

            if fields_str and fields_str[0] == ",":
                fields_str = fields_str[1:]

                # A comma can not exist immediately before another comma or the end of the string
                if not fields_str or fields_str[0] == ",":
                    raise FieldsParameterParseError(
                        "unexpected char ',' at position %d" % get_position(fields_str)
                    )

            is_first = False

        if expect_close_bracket:
            # This parser should've exited with a close bracket but instead we
            # hit the end of the input. Raise an error
            raise FieldsParameterParseError(
                "unexpected end of input (did you miss out a close bracket?)"
            )

        return fields, fields_str

    fields, _ = parse_fields(fields_str)

    return fields


def parse_boolean(value):
    """
    Parses strings into booleans using the following mapping (case-sensitive):

    'true'   => True
    'false'  => False
    '1'      => True
    '0'      => False
    """
    if value in ["true", "1"]:
        return True
    elif value in ["false", "0"]:
        return False
    else:
        raise ValueError("expected 'true' or 'false', got '%s'" % value)



from django.contrib.auth.models import Group, Permission
from django.test import TestCase

from wagtail.models import GroupPagePermission, Page
from wagtail.test.testapp.models import BusinessIndex, EventCategory, EventPage
from wagtail.test.utils import WagtailTestUtils


class TestIndexView(TestCase, WagtailTestUtils):
    fixtures = ["test_specific.json"]

    def setUp(self):
        self.login()

    def get(self, **params):
        return self.client.get("/admin/tests/eventpage/", params)

    def test_simple(self):
        response = self.get()

        self.assertEqual(response.status_code, 200)

        # There are four event pages in the test data
        self.assertEqual(response.context["result_count"], 4)

        # User has add permission
        self.assertIs(response.context["user_can_create"], True)

    def test_filter(self):
        # Filter by audience
        response = self.get(audience__exact="public")

        self.assertEqual(response.status_code, 200)

        # Only three of the event page in the test data are 'public'
        self.assertEqual(response.context["result_count"], 3)

        for eventpage in response.context["object_list"]:
            self.assertEqual(eventpage.audience, "public")

    def test_search(self):
        response = self.get(q="Someone")

        self.assertEqual(response.status_code, 200)

        # There is one eventpage where the title contains 'Someone'
        self.assertEqual(response.context["result_count"], 1)

    def test_ordering(self):
        response = self.get(o="0.1")

        self.assertEqual(response.status_code, 200)

        # There should still be four results
        self.assertEqual(response.context["result_count"], 4)

    def test_using_core_page(self):
        # The core page is slightly different to other pages, so exclude it
        response = self.client.get("/admin/wagtailcore/page/")
        self.assertEqual(response.status_code, 200)

        root_page = Page.objects.get(depth=1)
        self.assertNotIn(root_page, response.context["paginator"].object_list)


class TestExcludeFromExplorer(TestCase, WagtailTestUtils):
    fixtures = ["modeladmintest_test.json"]

    def setUp(self):
        self.login()

    def test_attribute_effects_explorer(self):
        # The two VenuePages should appear in the venuepage list
        response = self.client.get("/admin/modeladmintest/venuepage/")
        self.assertContains(response, "Santa&#x27;s Grotto")
        self.assertContains(response, "Santa&#x27;s Workshop")

        # But when viewing the children of 'Christmas' event in explorer
        response = self.client.get("/admin/pages/4/")
        self.assertNotContains(response, "Santa&#x27;s Grotto")
        self.assertNotContains(response, "Santa&#x27;s Workshop")

        # But the other test page should...
        self.assertContains(response, "Claim your free present!")


class TestCreateView(TestCase, WagtailTestUtils):
    fixtures = ["test_specific.json"]

    def setUp(self):
        self.login()

    def test_redirect_to_choose_parent(self):
        # When more than one possible parent page exists, redirect to choose_parent
        response = self.client.get("/admin/tests/eventpage/create/")
        self.assertRedirects(response, "/admin/tests/eventpage/choose_parent/")

    def test_one_parent_exists(self):
        # Create a BusinessIndex page that BusinessChild can exist under
        homepage = Page.objects.get(url_path="/home/")
        business_index = BusinessIndex(title="Business Index")
        homepage.add_child(instance=business_index)

        # When one possible parent page exists, redirect straight to the page create view
        response = self.client.get("/admin/tests/businesschild/create/")

        expected_path = "/admin/pages/add/tests/businesschild/%d/" % business_index.pk
        expected_next_path = "/admin/tests/businesschild/"
        self.assertRedirects(
            response, "%s?next=%s" % (expected_path, expected_next_path)
        )


class TestInspectView(TestCase, WagtailTestUtils):
    fixtures = ["test_specific.json", "modeladmintest_test.json"]

    def setUp(self):
        self.login()

    def get(self, id):
        return self.client.get("/admin/tests/eventpage/inspect/%d/" % id)

    def test_simple(self):
        response = self.get(4)
        self.assertEqual(response.status_code, 200)

    def test_title_present(self):
        """
        The page title should appear three times. Once in the header, and two times
        in the field listing (as the actual title and as the draft title)
        """
        response = self.get(4)
        self.assertContains(response, "Christmas", 3)

    def test_manytomany_output(self):
        """
        Because ManyToMany fields are output InspectView by default, the
        `categories` for the event should output as a comma separated list
        once populated.
        """
        eventpage = EventPage.objects.get(pk=4)
        free_category = EventCategory.objects.create(name="Free")
        child_friendly_category = EventCategory.objects.create(name="Child-friendly")
        eventpage.categories = (free_category, child_friendly_category)
        eventpage.save()
        response = self.get(4)
        self.assertContains(response, "<dd>Free, Child-friendly</dd>", html=True)

    def test_false_values_displayed(self):
        """
        Boolean fields with False values should display False, rather than the
        value of `get_empty_value_display()`. For this page, those should be
        `locked`, `expired` and `has_unpublished_changes`
        """
        response = self.get(4)
        self.assertContains(response, "<dd>False</dd>", count=3, html=True)

    def test_location_present(self):
        """
        The location should appear once, in the field listing
        """
        response = self.get(4)
        self.assertContains(response, "The North Pole", 1)

    def test_non_existent(self):
        response = self.get(100)
        self.assertEqual(response.status_code, 404)

    def test_short_description_is_used_as_field_label(self):
        """
        A custom field has been added to the inspect view's `inspect_view_fields` and since
        this field has a `short_description` we expect it to be used as the field's label,
        and not use the name of the function.
        """
        response = self.client.get("/admin/modeladmintest/author/inspect/1/")
        self.assertContains(response, "Birth information")
        self.assertNotContains(response, "author_birth_string")


class TestEditView(TestCase, WagtailTestUtils):
    fixtures = ["test_specific.json"]

    def setUp(self):
        self.login()

    def get(self, obj_id):
        return self.client.get("/admin/tests/eventpage/edit/%d/" % obj_id)

    def test_simple(self):
        response = self.get(4)

        expected_path = "/admin/pages/4/edit/"
        expected_next_path = "/admin/tests/eventpage/"
        self.assertRedirects(
            response, "%s?next=%s" % (expected_path, expected_next_path)
        )

    def test_non_existent(self):
        response = self.get(100)

        self.assertEqual(response.status_code, 404)

    def test_using_core_page(self):
        # The core page is slightly different to other pages, so exclude it
        root_page = Page.objects.get(depth=1)
        response = self.client.get("/admin/wagtailcore/page/{}/".format(root_page.id))
        self.assertEqual(response.status_code, 404)


class TestDeleteView(TestCase, WagtailTestUtils):
    fixtures = ["test_specific.json"]

    def setUp(self):
        self.login()

    def get(self, obj_id):
        return self.client.get("/admin/tests/eventpage/delete/%d/" % obj_id)

    def test_simple(self):
        response = self.get(4)

        expected_path = "/admin/pages/4/delete/"
        expected_next_path = "/admin/tests/eventpage/"
        self.assertRedirects(
            response, "%s?next=%s" % (expected_path, expected_next_path)
        )


class TestChooseParentView(TestCase, WagtailTestUtils):
    fixtures = ["test_specific.json"]

    def setUp(self):
        self.login()

    def test_simple(self):
        response = self.client.get("/admin/tests/eventpage/choose_parent/")

        self.assertEqual(response.status_code, 200)

    def test_no_parent_exists(self):
        response = self.client.get("/admin/tests/businesschild/choose_parent/")

        self.assertRedirects(response, "/admin/")

    def test_post(self):
        response = self.client.post(
            "/admin/tests/eventpage/choose_parent/",
            {
                "parent_page": 2,
            },
        )

        expected_path = "/admin/pages/add/tests/eventpage/2/"
        expected_next_path = "/admin/tests/eventpage/"
        self.assertRedirects(
            response, "%s?next=%s" % (expected_path, expected_next_path)
        )


class TestChooseParentViewForNonSuperuser(TestCase, WagtailTestUtils):
    fixtures = ["test_specific.json"]

    def setUp(self):
        homepage = Page.objects.get(url_path="/home/")
        business_index = BusinessIndex(
            title="Public Business Index",
            draft_title="Public Business Index",
        )
        homepage.add_child(instance=business_index)

        another_business_index = BusinessIndex(
            title="Another Business Index",
            draft_title="Another Business Index",
        )
        homepage.add_child(instance=another_business_index)

        secret_business_index = BusinessIndex(
            title="Private Business Index",
            draft_title="Private Business Index",
        )
        homepage.add_child(instance=secret_business_index)

        business_editors = Group.objects.create(name="Business editors")
        business_editors.permissions.add(
            Permission.objects.get(codename="access_admin")
        )
        GroupPagePermission.objects.create(
            group=business_editors, page=business_index, permission_type="add"
        )
        GroupPagePermission.objects.create(
            group=business_editors, page=another_business_index, permission_type="add"
        )

        user = self.create_user(username="test2", password="password")
        user.groups.add(business_editors)
        # Login
        self.login(username="test2", password="password")

    def test_simple(self):
        response = self.client.get("/admin/tests/businesschild/choose_parent/")

        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "Public Business Index")
        self.assertNotContains(response, "Private Business Index")


class TestEditorAccess(TestCase, WagtailTestUtils):
    fixtures = ["test_specific.json"]

    def setUp(self):
        # Create a user
        user = self.create_user(username="test2", password="password")
        user.groups.add(Group.objects.get(pk=2))
        # Login
        self.login(username="test2", password="password")

    def test_delete_permitted(self):
        response = self.client.get("/admin/tests/eventpage/delete/4/")
        self.assertRedirects(response, "/admin/")


class TestModeratorAccess(TestCase, WagtailTestUtils):
    fixtures = ["test_specific.json"]

    def setUp(self):
        # Create a user
        user = self.create_user(username="test3", password="password")
        user.groups.add(Group.objects.get(pk=1))
        # Login
        self.login(username="test3", password="password")

    def test_delete_permitted(self):
        response = self.client.get("/admin/tests/eventpage/delete/4/")
        self.assertRedirects(
            response, "/admin/pages/4/delete/?next=/admin/tests/eventpage/"
        )


class TestHeaderBreadcrumbs(TestCase, WagtailTestUtils):
    """
    Test that the breadcrumbs region is inserted before the
    </header> tag for potential future regression.
    See https://github.com/wagtail/wagtail/issues/3889
    """

    fixtures = ["test_specific.json"]

    def setUp(self):
        self.login()

    def test_choose_parent_page(self):
        response = self.client.get("/admin/tests/eventpage/choose_parent/")

        # check correct templates were used
        self.assertTemplateUsed(response, "modeladmin/includes/breadcrumb.html")
        self.assertTemplateUsed(response, "wagtailadmin/shared/header.html")

        # check that home breadcrumb link exists
        expected = """
            <li class="breadcrumb-item home">
                <a href="/admin/" class="breadcrumb-link">
                    <svg class="icon icon-home home_icon" aria-hidden="true">
                        <use href="#icon-home"></use>
                    </svg>
                    <span class="visuallyhidden">Home</span>
                    <svg class="icon icon-arrow-right arrow_right_icon" aria-hidden="true">
                        <use href="#icon-arrow-right"></use>
                    </svg>
                </a>
            </li>
        """
        self.assertContains(response, expected, html=True)

        # check that the breadcrumbs are before the first header closing tag
        content_str = str(response.content)
        position_of_header_close = content_str.index("</header>")
        position_of_breadcrumbs = content_str.index('<ul class="breadcrumb">')
        self.assertGreater(position_of_header_close, position_of_breadcrumbs)

    def test_choose_inspect_page(self):
        response = self.client.get("/admin/tests/eventpage/inspect/4/")

        # check correct templates were used
        self.assertTemplateUsed(response, "modeladmin/includes/breadcrumb.html")
        self.assertTemplateUsed(response, "wagtailadmin/shared/header.html")

        # check that home breadcrumb link exists
        expected = """
            <li class="breadcrumb-item home">
                <a href="/admin/" class="breadcrumb-link">
                    <svg class="icon icon-home home_icon" aria-hidden="true">
                        <use href="#icon-home"></use>
                    </svg>
                    <span class="visuallyhidden">Home</span>
                    <svg class="icon icon-arrow-right arrow_right_icon" aria-hidden="true">
                        <use href="#icon-arrow-right"></use>
                    </svg>
                </a>
            </li>
        """
        self.assertContains(response, expected, html=True)

        # check that the breadcrumbs are before the first header closing tag
        content_str = str(response.content)
        position_of_header_close = content_str.index("</header>")
        position_of_breadcrumbs = content_str.index('<ul class="breadcrumb">')
        self.assertGreater(position_of_header_close, position_of_breadcrumbs)


class TestSearch(TestCase, WagtailTestUtils):
    fixtures = ["test_specific.json"]

    def setUp(self):
        self.login()

    def test_lookup_allowed_on_parentalkey(self):
        try:
            self.client.get(
                "/admin/tests/eventpage/?related_links__link_page__id__exact=1"
            )
        except AttributeError:
            self.fail("Lookup on parentalkey raised AttributeError unexpectedly")



from django.core.management.base import BaseCommand
from django.utils import dateparse, timezone

from wagtail.models import Page, PageRevision


def revision_date_expired(r):
    expiry_str = r.content.get("expire_at")
    if not expiry_str:
        return False
    expire_at = dateparse.parse_datetime(expiry_str)
    if expire_at < timezone.now():
        return True
    else:
        return False


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument(
            "--dryrun",
            action="store_true",
            dest="dryrun",
            default=False,
            help="Dry run -- don't change anything.",
        )

    def handle(self, *args, **options):
        dryrun = False
        if options["dryrun"]:
            self.stdout.write("Will do a dry run.")
            dryrun = True

        # 1. get all expired pages with live = True
        expired_pages = Page.objects.filter(live=True, expire_at__lt=timezone.now())
        if dryrun:
            if expired_pages:
                self.stdout.write("Expired pages to be deactivated:")
                self.stdout.write("Expiry datetime\t\tSlug\t\tName")
                self.stdout.write("---------------\t\t----\t\t----")
                for ep in expired_pages:
                    self.stdout.write(
                        "{0}\t{1}\t{2}".format(
                            ep.expire_at.strftime("%Y-%m-%d %H:%M"), ep.slug, ep.title
                        )
                    )
            else:
                self.stdout.write("No expired pages to be deactivated found.")
        else:
            # Unpublish the expired pages
            # Cast to list to make sure the query is fully evaluated
            # before unpublishing anything
            for page in list(expired_pages):
                page.unpublish(
                    set_expired=True, log_action="wagtail.unpublish.scheduled"
                )

        # 2. get all page revisions for moderation that have been expired
        expired_revs = [
            r
            for r in PageRevision.objects.filter(submitted_for_moderation=True)
            if revision_date_expired(r)
        ]
        if dryrun:
            self.stdout.write("---------------------------------")
            if expired_revs:
                self.stdout.write(
                    "Expired revisions to be dropped from moderation queue:"
                )
                self.stdout.write("Expiry datetime\t\tSlug\t\tName")
                self.stdout.write("---------------\t\t----\t\t----")
                for er in expired_revs:
                    rev_data = er.content
                    self.stdout.write(
                        "{0}\t{1}\t{2}".format(
                            dateparse.parse_datetime(
                                rev_data.get("expire_at")
                            ).strftime("%Y-%m-%d %H:%M"),
                            rev_data.get("slug"),
                            rev_data.get("title"),
                        )
                    )
            else:
                self.stdout.write("No expired revision to be dropped from moderation.")
        else:
            for er in expired_revs:
                er.submitted_for_moderation = False
                er.save()

        # 3. get all revisions that need to be published
        revs_for_publishing = PageRevision.objects.filter(
            approved_go_live_at__lt=timezone.now()
        )
        if dryrun:
            self.stdout.write("---------------------------------")
            if revs_for_publishing:
                self.stdout.write("Revisions to be published:")
                self.stdout.write("Go live datetime\t\tSlug\t\tName")
                self.stdout.write("---------------\t\t\t----\t\t----")
                for rp in revs_for_publishing:
                    rev_data = rp.content
                    self.stdout.write(
                        "{0}\t\t{1}\t{2}".format(
                            rp.approved_go_live_at.strftime("%Y-%m-%d %H:%M"),
                            rev_data.get("slug"),
                            rev_data.get("title"),
                        )
                    )
            else:
                self.stdout.write("No pages to go live.")
        else:
            for rp in revs_for_publishing:
                # just run publish for the revision -- since the approved go
                # live datetime is before now it will make the page live
                rp.publish(log_action="wagtail.publish.scheduled")



# -*- coding: utf-8 -*-
from django.db import migrations, models
import modelcluster.fields
import wagtail.fields


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailcore", "0028_merge"),
        (
            "tests",
            "0005_customrichblockfieldpage_customrichtextfieldpage_defaultrichblockfieldpage_defaultrichtextfieldpage",
        ),
    ]

    operations = [
        migrations.CreateModel(
            name="SectionedRichTextPage",
            fields=[
                (
                    "page_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        to="wagtailcore.Page",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                        on_delete=models.CASCADE,
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("wagtailcore.page",),
        ),
        migrations.CreateModel(
            name="SectionedRichTextPageSection",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        primary_key=True,
                        serialize=False,
                        auto_created=True,
                    ),
                ),
                (
                    "sort_order",
                    models.IntegerField(editable=False, null=True, blank=True),
                ),
                ("body", wagtail.fields.RichTextField()),
                (
                    "page",
                    modelcluster.fields.ParentalKey(
                        related_name="sections",
                        to="tests.SectionedRichTextPage",
                        on_delete=models.CASCADE,
                    ),
                ),
            ],
            options={
                "ordering": ["sort_order"],
                "abstract": False,
            },
        ),
    ]



from wagtail import hooks


class HookResponseMixin:
    """
    A mixin for class-based views to support hooks like `before_edit_page`, which are triggered
    during execution of some operation and can return a response to halt that operation.
    """

    def run_hook(self, hook_name, *args, **kwargs):
        """
        Run the named hook, passing args and kwargs to each function registered under that hook name.
        If any return an HttpResponse, stop processing and return that response
        """
        for fn in hooks.get_hooks(hook_name):
            result = fn(*args, **kwargs)
            if hasattr(result, "status_code"):
                return result



from django.db import migrations
from django.db.models import Count, Min

from wagtail.embeds.embeds import get_embed_hash


def migrate_forwards(apps, schema_editor):
    Embed = apps.get_model("wagtailembeds.Embed")

    # based each embed being ~500b, available memory being
    # to 2MB, and leaving plenty of headway for app itself
    batch_size = 1500

    # allows small batches to be kept in memory in order
    # to utilise bulk_update()
    batch = []

    for embed in Embed.objects.all().only("id", "url", "max_width").iterator():

        embed.hash = get_embed_hash(embed.url, embed.max_width)
        batch.append(embed)

        if len(batch) == batch_size:
            # save and reset current batch
            Embed.objects.bulk_update(batch, ["hash"])
            batch.clear()

    # save any leftovers
    if batch:
        Embed.objects.bulk_update(batch, ["hash"])

    # delete duplicates
    duplicates = (
        Embed.objects.values("hash")
        .annotate(hash_count=Count("id"), min_id=Min("id"))
        .filter(hash_count__gt=1)
    )
    for dup in duplicates:
        # for each duplicated hash, delete all except the one with the lowest id
        Embed.objects.filter(hash=dup["hash"]).exclude(id=dup["min_id"]).delete()


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailembeds", "0006_add_embed_hash"),
    ]

    operations = [
        migrations.RunPython(migrate_forwards, migrations.RunPython.noop),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-05-12 20:56
from django.db import migrations, models
import django.db.models.deletion
import wagtail.blocks
import wagtail.fields


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailcore", "0028_merge"),
        ("tests", "0004_fileuploadsetting"),
    ]

    operations = [
        migrations.CreateModel(
            name="CustomRichBlockFieldPage",
            fields=[
                (
                    "page_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="wagtailcore.Page",
                    ),
                ),
                (
                    "body",
                    wagtail.fields.StreamField(
                        [
                            (
                                "rich_text",
                                wagtail.blocks.RichTextBlock(editor="custom"),
                            )
                        ]
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("wagtailcore.page",),
        ),
        migrations.CreateModel(
            name="CustomRichTextFieldPage",
            fields=[
                (
                    "page_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="wagtailcore.Page",
                    ),
                ),
                ("body", wagtail.fields.RichTextField()),
            ],
            options={
                "abstract": False,
            },
            bases=("wagtailcore.page",),
        ),
        migrations.CreateModel(
            name="DefaultRichBlockFieldPage",
            fields=[
                (
                    "page_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="wagtailcore.Page",
                    ),
                ),
                (
                    "body",
                    wagtail.fields.StreamField(
                        [("rich_text", wagtail.blocks.RichTextBlock())]
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("wagtailcore.page",),
        ),
        migrations.CreateModel(
            name="DefaultRichTextFieldPage",
            fields=[
                (
                    "page_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="wagtailcore.Page",
                    ),
                ),
                ("body", wagtail.fields.RichTextField()),
            ],
            options={
                "abstract": False,
            },
            bases=("wagtailcore.page",),
        ),
    ]



# -*- coding: utf-8 -*-
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("customuser", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="CustomUser",
            name="attachment",
            field=models.FileField(blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
import datetime
from io import BytesIO

from django.conf import settings
from django.contrib.auth.models import AnonymousUser
from django.test import RequestFactory, TestCase, override_settings
from django.urls import reverse
from openpyxl import load_workbook

from wagtail.admin.forms import WagtailAdminPageForm
from wagtail.admin.panels import get_form_for_model
from wagtail.contrib.forms.models import FormSubmission
from wagtail.contrib.forms.panels import FormSubmissionsPanel
from wagtail.contrib.forms.tests.utils import (
    make_form_page,
    make_form_page_with_custom_submission,
)
from wagtail.models import Locale, Page
from wagtail.test.testapp.models import (
    CustomFormPageSubmission,
    ExtendedFormField,
    FormField,
    FormFieldForCustomListViewPage,
    FormFieldWithCustomSubmission,
    FormPage,
    FormPageWithCustomFormBuilder,
    FormPageWithCustomSubmission,
    FormPageWithCustomSubmissionListView,
)
from wagtail.test.utils import WagtailTestUtils


class TestFormResponsesPanel(TestCase):
    def setUp(self):
        self.request = RequestFactory().get("/")
        user = AnonymousUser()  # technically, Anonymous users cannot access the admin
        self.request.user = user

        self.form_page = make_form_page()

        self.FormPageForm = get_form_for_model(
            FormPage,
            form_class=WagtailAdminPageForm,
            fields=["title", "slug", "to_address", "from_address", "subject"],
        )

        panel = FormSubmissionsPanel().bind_to_model(FormPage)
        self.panel = panel.get_bound_panel(
            instance=self.form_page, form=self.FormPageForm(), request=self.request
        )

    def test_render_with_submissions(self):
        """Show the panel with the count of submission and a link to the list_submissions view."""
        self.client.post(
            "/contact-us/",
            {
                "your_email": "bob@example.com",
                "your_message": "hello world",
                "your_choices": {"foo": "", "bar": "", "baz": ""},
            },
        )

        self.assertTrue(self.panel.is_shown())
        result = self.panel.render_html()

        url = reverse("wagtailforms:list_submissions", args=(self.form_page.id,))
        link = '<a href="{}">1</a>'.format(url)

        self.assertIn(link, result)

    def test_render_without_submissions(self):
        """The panel should not be shown if the number of submission is zero."""
        self.assertFalse(self.panel.is_shown())


class TestFormResponsesPanelWithCustomSubmissionClass(TestCase, WagtailTestUtils):
    def setUp(self):
        self.request = RequestFactory().get("/")
        user = AnonymousUser()  # technically, Anonymous users cannot access the admin
        self.request.user = user

        # Create a form page
        self.form_page = make_form_page_with_custom_submission()

        self.FormPageForm = get_form_for_model(
            FormPageWithCustomSubmission,
            form_class=WagtailAdminPageForm,
            fields=["title", "slug", "to_address", "from_address", "subject"],
        )

        self.test_user = self.create_user(username="user-n1kola", password="123")

        panel = FormSubmissionsPanel().bind_to_model(FormPageWithCustomSubmission)
        self.panel = panel.get_bound_panel(
            instance=self.form_page, form=self.FormPageForm(), request=self.request
        )

    def test_render_with_submissions(self):
        """Show the panel with the count of submission and a link to the list_submissions view."""
        new_form_submission = CustomFormPageSubmission.objects.create(
            user=self.test_user,
            page=self.form_page,
            form_data={
                "your_email": "email@domain.com",
                "your_message": "hi joe",
                "your_choices": {"foo": "", "bar": "", "baz": ""},
            },
        )
        new_form_submission.submit_time = "2017-08-29T12:00:00.000Z"
        new_form_submission.save()

        self.assertTrue(self.panel.is_shown())
        result = self.panel.render_html()

        url = reverse("wagtailforms:list_submissions", args=(self.form_page.id,))
        link = '<a href="{}">1</a>'.format(url)

        self.assertIn(link, result)

    def test_render_without_submissions(self):
        """The panel should not be shown if the number of submission is zero."""
        self.assertFalse(self.panel.is_shown())


class TestFormsIndex(TestCase, WagtailTestUtils):
    fixtures = ["test.json"]

    def setUp(self):
        self.login(username="siteeditor", password="password")
        self.form_page = Page.objects.get(url_path="/home/contact-us/")

    def make_form_pages(self):
        """
        This makes 100 form pages and adds them as children to 'contact-us'
        This is used to test pagination on the forms index
        """
        for i in range(100):
            self.form_page.add_child(
                instance=FormPage(
                    title="Form " + str(i), slug="form-" + str(i), live=True
                )
            )

    def test_forms_index(self):
        response = self.client.get(reverse("wagtailforms:index"))

        # Check response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailforms/index.html")

    def test_forms_index_pagination(self):
        # Create some more form pages to make pagination kick in
        self.make_form_pages()

        # Get page two
        response = self.client.get(reverse("wagtailforms:index"), {"p": 2})

        # Check response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailforms/index.html")

        # Check that we got the correct page
        self.assertEqual(response.context["page_obj"].number, 2)

    def test_forms_index_pagination_invalid(self):
        # Create some more form pages to make pagination kick in
        self.make_form_pages()

        # Get page two
        response = self.client.get(reverse("wagtailforms:index"), {"p": "Hello world!"})

        # Check response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailforms/index.html")

        # Check that it got page one
        self.assertEqual(response.context["page_obj"].number, 1)

    def test_forms_index_pagination_out_of_range(self):
        # Create some more form pages to make pagination kick in
        self.make_form_pages()

        # Get page two
        response = self.client.get(reverse("wagtailforms:index"), {"p": 99999})

        # Check response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailforms/index.html")

        # Check that it got the last page
        self.assertEqual(
            response.context["page_obj"].number, response.context["paginator"].num_pages
        )

    def test_cannot_see_forms_without_permission(self):
        # Login with as a user without permission to see forms
        self.login(username="eventeditor", password="password")

        response = self.client.get(reverse("wagtailforms:index"))

        # Check that the user cannot see the form page
        self.assertNotIn(self.form_page, response.context["form_pages"])

    def test_can_see_forms_with_permission(self):
        response = self.client.get(reverse("wagtailforms:index"))

        # Check that the user can see the form page
        self.assertIn(self.form_page, response.context["form_pages"])

    def test_cant_see_forms_after_filter_form_submissions_for_user_hook(self):
        # Hook allows to see forms only to superusers
        def construct_forms_for_user(user, queryset):
            if not user.is_superuser:
                queryset = queryset.none()

            return queryset

        response = self.client.get(reverse("wagtailforms:index"))

        # Check that an user can see the form page
        self.assertIn(self.form_page, response.context["form_pages"])

        with self.register_hook(
            "filter_form_submissions_for_user", construct_forms_for_user
        ):
            response = self.client.get(reverse("wagtailforms:index"))

        # Check that an user can't see the form page
        self.assertNotIn(self.form_page, response.context["form_pages"])


@override_settings(WAGTAIL_I18N_ENABLED=True)
class TestFormsIndexWithLocalisationEnabled(TestCase, WagtailTestUtils):
    fixtures = ["test.json"]

    def setUp(self):
        self.login(username="superuser", password="password")
        self.form_page = Page.objects.get(url_path="/home/contact-us/")
        self.en_locale = Locale.get_default()

        self.fr_locale = Locale.objects.create(language_code="fr")
        self.fr_form_page = self.form_page.copy_for_translation(
            self.fr_locale, copy_parents=True
        )
        self.fr_form_page.save()

        self.forms_index_url = reverse("wagtailforms:index")

    def make_form_pages(self, num=100, parent=None):
        """
        This makes 100 form pages and adds them as children to 'contact-us'
        This is used to test pagination on the forms index
        """
        if parent is None:
            parent = self.form_page

        for i in range(num):
            suffix = f"{i} [{parent.locale.get_display_name()}]"
            parent.add_child(
                instance=FormPage(
                    title=f"Form {suffix}",
                    slug=f"form-{i}-{parent.locale_id}",
                    live=True,
                    locale_id=parent.locale_id,
                )
            )

    def get_switch_current_locale_markup(self, locale):
        return f'<a href="javascript:void(0)" aria-label="{locale.get_display_name()}" class="c-dropdown__button u-btn-current w-no-underline">'

    def get_switch_link_markup(self, locale):
        return f'<a href="{self.forms_index_url}?locale={locale.language_code}" aria-label="{locale.get_display_name()}" class="u-link is-live w-no-underline">'

    def test_forms_index(self):
        response = self.client.get(self.forms_index_url)

        # Check response
        self.assertEqual(response.status_code, 200)

        self.assertContains(
            response, self.get_switch_current_locale_markup(self.en_locale)
        )
        self.assertContains(response, self.get_switch_link_markup(self.fr_locale))

        response = self.client.get(
            self.forms_index_url, {"locale": self.fr_locale.language_code}
        )
        self.assertContains(
            response, self.get_switch_current_locale_markup(self.fr_locale)
        )
        self.assertContains(response, self.get_switch_link_markup(self.en_locale))

    def test_forms_index_pagination(self):
        # Create some more form pages to make pagination kick in
        self.make_form_pages(parent=self.form_page, num=20)
        self.make_form_pages(parent=self.fr_form_page, num=20)

        # Get page two
        response = self.client.get(self.forms_index_url, {"p": 2})

        # Check response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailforms/index.html")

        # Check that we got the correct page
        self.assertEqual(response.context["page_obj"].number, 2)

        response = self.client.get(self.forms_index_url, {"p": 3})
        self.assertEqual(response.context["page_obj"].number, 2)

        # now check the French pages.
        response = self.client.get(
            self.forms_index_url, {"p": 2, "locale": self.fr_locale.language_code}
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context["page_obj"].number, 2)

    @override_settings(WAGTAIL_I18N_ENABLED=False)
    def test_switcher_doesnt_show_with_i18n_disabled(self):
        response = self.client.get(self.forms_index_url)

        self.assertNotContains(
            response, self.get_switch_current_locale_markup(self.en_locale)
        )
        self.assertNotContains(response, self.get_switch_link_markup(self.fr_locale))


class TestFormsSubmissionsList(TestCase, WagtailTestUtils):
    def setUp(self):
        # Create a form page
        self.form_page = make_form_page()

        # Add a couple of form submissions
        # (save new_form_submission first, so that we're more likely to reveal bugs where
        # we're relying on the database's internal ordering instead of explicitly ordering
        # by submit_time)

        new_form_submission = FormSubmission.objects.create(
            page=self.form_page,
            form_data={
                "your_email": "new@example.com",
                "your_message": "this is a fairly new message",
                "your_choices": ["foo", "baz"],
            },
        )
        new_form_submission.submit_time = "2014-01-01T12:00:00.000Z"
        new_form_submission.save()

        old_form_submission = FormSubmission.objects.create(
            page=self.form_page,
            form_data={
                "your_email": "old@example.com",
                "your_message": "this is a really old message",
            },
        )
        old_form_submission.submit_time = "2013-01-01T12:00:00.000Z"
        old_form_submission.save()

        # Login
        self.login()

    def make_list_submissions(self):
        """
        This makes 100 submissions to test pagination on the forms submissions page
        """
        for i in range(100):
            submission = FormSubmission(
                page=self.form_page, form_data={"hello": "world"}
            )
            submission.save()

    def test_list_submissions(self):
        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,))
        )

        # Check response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailforms/index_submissions.html")
        self.assertEqual(len(response.context["data_rows"]), 2)

        # check display of list values within form submissions
        self.assertContains(response, "foo, baz")

    def test_list_submissions_after_filter_form_submissions_for_user_hook(self):
        # Hook forbids to delete form submissions for everyone
        def construct_forms_for_user(user, queryset):
            return queryset.none()

        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,))
        )

        # An user can see form submissions without the hook
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailforms/index_submissions.html")
        self.assertEqual(len(response.context["data_rows"]), 2)

        with self.register_hook(
            "filter_form_submissions_for_user", construct_forms_for_user
        ):
            response = self.client.get(
                reverse("wagtailforms:list_submissions", args=(self.form_page.id,))
            )

        # An user can't see form submissions with the hook
        self.assertRedirects(response, "/admin/")

    def test_list_submissions_filtering_date_from(self):
        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"date_from": "01/01/2014"},
        )

        # Check response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailforms/index_submissions.html")
        self.assertEqual(len(response.context["data_rows"]), 1)

    def test_list_submissions_filtering_date_to(self):
        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"date_to": "12/31/2013"},
        )

        # Check response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailforms/index_submissions.html")
        self.assertEqual(len(response.context["data_rows"]), 1)

    def test_list_submissions_filtering_range(self):
        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"date_from": "12/31/2013", "date_to": "01/02/2014"},
        )

        # Check response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailforms/index_submissions.html")
        self.assertEqual(len(response.context["data_rows"]), 1)

    def test_list_submissions_pagination(self):
        self.make_list_submissions()

        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"p": 2},
        )

        # Check response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailforms/index_submissions.html")

        # Check that we got the correct page
        self.assertEqual(response.context["page_obj"].number, 2)

    def test_list_submissions_pagination_invalid(self):
        self.make_list_submissions()

        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"p": "Hello World!"},
        )

        # Check response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailforms/index_submissions.html")

        # Check that we got page one
        self.assertEqual(response.context["page_obj"].number, 1)

    def test_list_submissions_pagination_out_of_range(self):
        self.make_list_submissions()

        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"p": 99999},
        )

        # Check response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailforms/index_submissions.html")

        # Check that we got the last page
        self.assertEqual(
            response.context["page_obj"].number, response.context["paginator"].num_pages
        )

    def test_list_submissions_default_order(self):
        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,))
        )
        # check default ordering, most recent responses first
        first_row_values = response.context["data_rows"][0]["fields"]
        self.assertIn("this is a fairly new message", first_row_values)

    def test_list_submissions_url_params_ordering_recent_first(self):
        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"order_by": "-submit_time"},
        )
        # check ordering matches '-submit_time' (most recent first)
        first_row_values = response.context["data_rows"][0]["fields"]
        self.assertIn("this is a fairly new message", first_row_values)

    def test_list_submissions_url_params_ordering_oldest_first(self):
        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"order_by": "submit_time"},
        )
        # check ordering matches 'submit_time' (oldest first)
        first_row_values = response.context["data_rows"][0]["fields"]
        self.assertIn("this is a really old message", first_row_values)


class TestFormsSubmissionsExport(TestCase, WagtailTestUtils):
    def setUp(self):
        # Create a form page
        self.form_page = make_form_page()

        # Add a couple of form submissions
        old_form_submission = FormSubmission.objects.create(
            page=self.form_page,
            form_data={
                "your_email": "old@example.com",
                "your_message": "this is a really old message",
                "your_choices": ["foo", "baz"],
            },
        )
        if settings.USE_TZ:
            old_form_submission.submit_time = "2013-01-01T12:00:00.000Z"
        else:
            old_form_submission.submit_time = "2013-01-01T12:00:00"
        old_form_submission.save()

        new_form_submission = FormSubmission.objects.create(
            page=self.form_page,
            form_data={
                "your_email": "new@example.com",
                "your_message": "this is a fairly new message",
            },
        )
        if settings.USE_TZ:
            new_form_submission.submit_time = "2014-01-01T12:00:00.000Z"
        else:
            new_form_submission.submit_time = "2014-01-01T12:00:00"
        new_form_submission.save()

        # Login
        self.login()

    def test_list_submissions_csv_export(self):
        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"export": "csv"},
        )

        # Check response
        self.assertEqual(response.status_code, 200)
        data_lines = response.getvalue().decode().split("\n")

        self.assertEqual(
            data_lines[0], "Submission date,Your email,Your message,Your choices\r"
        )
        if settings.USE_TZ:
            self.assertEqual(
                data_lines[1],
                '2013-01-01 12:00:00+00:00,old@example.com,this is a really old message,"foo, baz"\r',
            )
            self.assertEqual(
                data_lines[2],
                "2014-01-01 12:00:00+00:00,new@example.com,this is a fairly new message,None\r",
            )
        else:
            self.assertEqual(
                data_lines[1],
                '2013-01-01 12:00:00,old@example.com,this is a really old message,"foo, baz"\r',
            )
            self.assertEqual(
                data_lines[2],
                "2014-01-01 12:00:00,new@example.com,this is a fairly new message,None\r",
            )

    def test_list_submissions_xlsx_export(self):
        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"export": "xlsx"},
        )

        self.assertEqual(response.status_code, 200)
        workbook_data = response.getvalue()
        worksheet = load_workbook(filename=BytesIO(workbook_data))["Sheet1"]
        cell_array = [[cell.value for cell in row] for row in worksheet.rows]
        self.assertEqual(
            cell_array[0],
            ["Submission date", "Your email", "Your message", "Your choices"],
        )
        self.assertEqual(
            cell_array[1],
            [
                datetime.datetime(2013, 1, 1, 12, 0),
                "old@example.com",
                "this is a really old message",
                "foo, baz",
            ],
        )
        self.assertEqual(
            cell_array[2],
            [
                datetime.datetime(2014, 1, 1, 12, 0),
                "new@example.com",
                "this is a fairly new message",
                "None",
            ],
        )
        self.assertEqual(len(cell_array), 3)

    def test_list_submissions_csv_large_export(self):
        for i in range(100):
            new_form_submission = FormSubmission.objects.create(
                page=self.form_page,
                form_data={
                    "your-email": "new@example-%s.com" % i,
                    "your-message": "I like things x %s" % i,
                },
            )
            if settings.USE_TZ:
                new_form_submission.submit_time = "2014-01-01T12:00:00.000Z"
            else:
                new_form_submission.submit_time = "2014-01-01T12:00:00"
            new_form_submission.save()

        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"export": "csv"},
        )

        # Check that csv export is not paginated
        self.assertEqual(response.status_code, 200)
        data_lines = response.getvalue().decode().split("\n")
        self.assertEqual(104, len(data_lines))

    def test_list_submissions_csv_export_after_filter_form_submissions_for_user_hook(
        self,
    ):
        # Hook forbids to delete form submissions for everyone
        def construct_forms_for_user(user, queryset):
            return queryset.none()

        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"export": "csv"},
        )

        # An user can export form submissions without the hook
        self.assertEqual(response.status_code, 200)
        data_lines = response.getvalue().decode().split("\n")

        self.assertEqual(
            data_lines[0], "Submission date,Your email,Your message,Your choices\r"
        )
        if settings.USE_TZ:
            self.assertEqual(
                data_lines[1],
                '2013-01-01 12:00:00+00:00,old@example.com,this is a really old message,"foo, baz"\r',
            )
            self.assertEqual(
                data_lines[2],
                "2014-01-01 12:00:00+00:00,new@example.com,this is a fairly new message,None\r",
            )
        else:
            self.assertEqual(
                data_lines[1],
                '2013-01-01 12:00:00,old@example.com,this is a really old message,"foo, baz"\r',
            )
            self.assertEqual(
                data_lines[2],
                "2014-01-01 12:00:00,new@example.com,this is a fairly new message,None\r",
            )

        with self.register_hook(
            "filter_form_submissions_for_user", construct_forms_for_user
        ):
            response = self.client.get(
                reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
                {"export": "csv"},
            )

        # An user can't export form submission with the hook
        self.assertRedirects(response, "/admin/")

    def test_list_submissions_csv_export_with_date_from_filtering(self):
        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"export": "csv", "date_from": "01/01/2014"},
        )

        # Check response
        self.assertEqual(response.status_code, 200)
        data_lines = response.getvalue().decode().split("\n")

        self.assertEqual(
            data_lines[0], "Submission date,Your email,Your message,Your choices\r"
        )
        if settings.USE_TZ:
            self.assertEqual(
                data_lines[1],
                "2014-01-01 12:00:00+00:00,new@example.com,this is a fairly new message,None\r",
            )
        else:
            self.assertEqual(
                data_lines[1],
                "2014-01-01 12:00:00,new@example.com,this is a fairly new message,None\r",
            )

    def test_list_submissions_csv_export_with_date_to_filtering(self):
        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"export": "csv", "date_to": "12/31/2013"},
        )

        # Check response
        self.assertEqual(response.status_code, 200)
        data_lines = response.getvalue().decode().split("\n")

        self.assertEqual(
            data_lines[0], "Submission date,Your email,Your message,Your choices\r"
        )
        if settings.USE_TZ:
            self.assertEqual(
                data_lines[1],
                '2013-01-01 12:00:00+00:00,old@example.com,this is a really old message,"foo, baz"\r',
            )
        else:
            self.assertEqual(
                data_lines[1],
                '2013-01-01 12:00:00,old@example.com,this is a really old message,"foo, baz"\r',
            )

    def test_list_submissions_csv_export_with_range_filtering(self):
        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"export": "csv", "date_from": "12/31/2013", "date_to": "01/02/2014"},
        )

        # Check response
        self.assertEqual(response.status_code, 200)
        data_lines = response.getvalue().decode().split("\n")

        self.assertEqual(
            data_lines[0], "Submission date,Your email,Your message,Your choices\r"
        )
        if settings.USE_TZ:
            self.assertEqual(
                data_lines[1],
                "2014-01-01 12:00:00+00:00,new@example.com,this is a fairly new message,None\r",
            )
        else:
            self.assertEqual(
                data_lines[1],
                "2014-01-01 12:00:00,new@example.com,this is a fairly new message,None\r",
            )

    def test_list_submissions_csv_export_with_unicode_in_submission(self):
        unicode_form_submission = FormSubmission.objects.create(
            page=self.form_page,
            form_data={
                "your_email": "unicode@example.com",
                "your_message": "こんにちは、世界",
            },
        )
        unicode_form_submission.submit_time = "2014-01-02T12:00:00.000Z"
        unicode_form_submission.save()

        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"date_from": "01/02/2014", "export": "csv"},
        )

        # Check response
        self.assertEqual(response.status_code, 200)
        data_line = response.getvalue().decode("utf-8").split("\n")[1]
        self.assertIn("こんにちは、世界", data_line)

    def test_list_submissions_csv_export_with_unicode_in_field(self):
        FormField.objects.create(
            page=self.form_page,
            sort_order=2,
            label="Выберите самую любимую IDE для разработке на Python",
            help_text="Вы можете выбрать только один вариант",
            field_type="radio",
            required=True,
            choices="PyCharm,vim,nano",
        )
        unicode_form_submission = FormSubmission.objects.create(
            page=self.form_page,
            form_data={
                "your_email": "unicode@example.com",
                "your_message": "We don't need unicode here",
                "u0412u044bu0431u0435u0440u0438u0442u0435_u0441u0430u043cu0443u044e_u043bu044eu0431u0438u043cu0443u044e_ide_u0434u043bu044f_u0440u0430u0437u0440u0430u0431u043eu0442u043au0435_u043du0430_python": "vim",
            },
        )
        unicode_form_submission.submit_time = "2014-01-02T12:00:00.000Z"
        unicode_form_submission.save()

        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"date_from": "01/02/2014", "export": "csv"},
        )

        # Check response
        self.assertEqual(response.status_code, 200)

        data_lines = response.getvalue().decode("utf-8").split("\n")
        self.assertIn(
            "Выберите самую любимую IDE для разработке на Python", data_lines[0]
        )
        self.assertIn("vim", data_lines[1])


class TestCustomFormsSubmissionsExport(TestCase, WagtailTestUtils):
    def create_test_user_without_admin(self, username):
        return self.create_user(username=username, password="123")

    def setUp(self):
        # Create a form page
        self.form_page = make_form_page_with_custom_submission()

        # Add a couple of form submissions
        old_form_submission = CustomFormPageSubmission.objects.create(
            user=self.create_test_user_without_admin("user-john"),
            page=self.form_page,
            form_data={
                "your_email": "old@example.com",
                "your_message": "this is a really old message",
            },
        )
        if settings.USE_TZ:
            old_form_submission.submit_time = "2013-01-01T12:00:00.000Z"
        else:
            old_form_submission.submit_time = "2013-01-01T12:00:00"
        old_form_submission.save()

        new_form_submission = CustomFormPageSubmission.objects.create(
            user=self.create_test_user_without_admin("user-m1kola"),
            page=self.form_page,
            form_data={
                "your_email": "new@example.com",
                "your_message": "this is a fairly new message",
            },
        )
        if settings.USE_TZ:
            new_form_submission.submit_time = "2014-01-01T12:00:00.000Z"
        else:
            new_form_submission.submit_time = "2014-01-01T12:00:00"
        new_form_submission.save()

        # Login
        self.login()

    def test_list_submissions_csv_export(self):
        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"export": "csv"},
        )

        # Check response
        self.assertEqual(response.status_code, 200)
        data_lines = response.getvalue().decode().split("\n")

        self.assertEqual(
            data_lines[0],
            "User email,Submission date,Your email,Your message,Your choices\r",
        )
        if settings.USE_TZ:
            self.assertEqual(
                data_lines[1],
                "user-john@example.com,2013-01-01 12:00:00+00:00,old@example.com,this is a really old message,None\r",
            )
            self.assertEqual(
                data_lines[2],
                "user-m1kola@example.com,2014-01-01 12:00:00+00:00,new@example.com,this is a fairly new message,None\r",
            )
        else:
            self.assertEqual(
                data_lines[1],
                "user-john@example.com,2013-01-01 12:00:00,old@example.com,this is a really old message,None\r",
            )
            self.assertEqual(
                data_lines[2],
                "user-m1kola@example.com,2014-01-01 12:00:00,new@example.com,this is a fairly new message,None\r",
            )

    def test_list_submissions_csv_export_with_date_from_filtering(self):
        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"export": "csv", "date_from": "01/01/2014"},
        )

        # Check response
        self.assertEqual(response.status_code, 200)
        data_lines = response.getvalue().decode().split("\n")

        self.assertEqual(
            data_lines[0],
            "User email,Submission date,Your email,Your message,Your choices\r",
        )
        if settings.USE_TZ:
            self.assertEqual(
                data_lines[1],
                "user-m1kola@example.com,2014-01-01 12:00:00+00:00,new@example.com,this is a fairly new message,None\r",
            )
        else:
            self.assertEqual(
                data_lines[1],
                "user-m1kola@example.com,2014-01-01 12:00:00,new@example.com,this is a fairly new message,None\r",
            )

    def test_list_submissions_csv_export_with_date_to_filtering(self):
        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"export": "csv", "date_to": "12/31/2013"},
        )

        # Check response
        self.assertEqual(response.status_code, 200)
        data_lines = response.getvalue().decode().split("\n")

        self.assertEqual(
            data_lines[0],
            "User email,Submission date,Your email,Your message,Your choices\r",
        )
        if settings.USE_TZ:
            self.assertEqual(
                data_lines[1],
                "user-john@example.com,2013-01-01 12:00:00+00:00,old@example.com,this is a really old message,None\r",
            )
        else:
            self.assertEqual(
                data_lines[1],
                "user-john@example.com,2013-01-01 12:00:00,old@example.com,this is a really old message,None\r",
            )

    def test_list_submissions_csv_export_with_range_filtering(self):
        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"export": "csv", "date_from": "12/31/2013", "date_to": "01/02/2014"},
        )

        # Check response
        self.assertEqual(response.status_code, 200)
        data_lines = response.getvalue().decode().split("\n")

        self.assertEqual(
            data_lines[0],
            "User email,Submission date,Your email,Your message,Your choices\r",
        )
        if settings.USE_TZ:
            self.assertEqual(
                data_lines[1],
                "user-m1kola@example.com,2014-01-01 12:00:00+00:00,new@example.com,this is a fairly new message,None\r",
            )
        else:
            self.assertEqual(
                data_lines[1],
                "user-m1kola@example.com,2014-01-01 12:00:00,new@example.com,this is a fairly new message,None\r",
            )

    def test_list_submissions_csv_export_with_unicode_in_submission(self):
        unicode_form_submission = CustomFormPageSubmission.objects.create(
            user=self.create_test_user_without_admin("user-bob"),
            page=self.form_page,
            form_data={
                "your_email": "unicode@example.com",
                "your_message": "こんにちは、世界",
            },
        )
        unicode_form_submission.submit_time = "2014-01-02T12:00:00.000Z"
        unicode_form_submission.save()

        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"date_from": "01/02/2014", "export": "csv"},
        )

        # Check response
        self.assertEqual(response.status_code, 200)
        data_line = response.getvalue().decode("utf-8").split("\n")[1]
        self.assertIn("こんにちは、世界", data_line)

    def test_list_submissions_csv_export_with_unicode_in_field(self):
        FormFieldWithCustomSubmission.objects.create(
            page=self.form_page,
            sort_order=2,
            label="Выберите самую любимую IDE для разработке на Python",
            help_text="Вы можете выбрать только один вариант",
            field_type="radio",
            required=True,
            choices="PyCharm,vim,nano",
        )
        unicode_form_submission = CustomFormPageSubmission.objects.create(
            user=self.create_test_user_without_admin("user-bob"),
            page=self.form_page,
            form_data={
                "your-email": "unicode@example.com",
                "your-message": "We don't need unicode here",
                "u0412u044bu0431u0435u0440u0438u0442u0435_u0441u0430u043cu0443u044e_u043bu044eu0431u0438u043cu0443u044e_ide_u0434u043bu044f_u0440u0430u0437u0440u0430u0431u043eu0442u043au0435_u043du0430_python": "vim",
            },
        )
        unicode_form_submission.submit_time = "2014-01-02T12:00:00.000Z"
        unicode_form_submission.save()

        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"date_from": "01/02/2014", "export": "csv"},
        )

        # Check response
        self.assertEqual(response.status_code, 200)

        data_lines = response.getvalue().decode("utf-8").split("\n")
        self.assertIn(
            "Выберите самую любимую IDE для разработке на Python", data_lines[0]
        )
        self.assertIn("vim", data_lines[1])


class TestCustomFormsSubmissionsList(TestCase, WagtailTestUtils):
    def create_test_user_without_admin(self, username):
        return self.create_user(username=username, password="123")

    def setUp(self):
        # Create a form page
        self.form_page = make_form_page_with_custom_submission()

        # Add a couple of form submissions
        old_form_submission = CustomFormPageSubmission.objects.create(
            user=self.create_test_user_without_admin("user-john"),
            page=self.form_page,
            form_data={
                "your_email": "old@example.com",
                "your_message": "this is a really old message",
            },
        )
        old_form_submission.submit_time = "2013-01-01T12:00:00.000Z"
        old_form_submission.save()

        new_form_submission = CustomFormPageSubmission.objects.create(
            user=self.create_test_user_without_admin("user-m1kola"),
            page=self.form_page,
            form_data={
                "your_email": "new@example.com",
                "your_message": "this is a fairly new message",
            },
        )
        new_form_submission.submit_time = "2014-01-01T12:00:00.000Z"
        new_form_submission.save()

        # Login
        self.login()

    def make_list_submissions(self):
        """
        This makes 100 submissions to test pagination on the forms submissions page
        """
        for i in range(100):
            submission = CustomFormPageSubmission(
                user=self.create_test_user_without_admin("generated-username-%s" % i),
                page=self.form_page,
                form_data={
                    "your_email": "generated-your-email-%s" % i,
                    "your_message": "generated-your-message-%s" % i,
                },
            )
            submission.save()

    def test_list_submissions(self):
        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,))
        )

        # Check response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailforms/index_submissions.html")
        self.assertEqual(len(response.context["data_rows"]), 2)

        # CustomFormPageSubmission have custom field. This field should appear in the listing
        self.assertContains(
            response, '<th id="useremail" class="">User email</th>', html=True
        )
        self.assertContains(response, "<td>user-m1kola@example.com</td>", html=True)
        self.assertContains(response, "<td>user-john@example.com</td>", html=True)

    def test_list_submissions_filtering_date_from(self):
        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"date_from": "01/01/2014"},
        )

        # Check response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailforms/index_submissions.html")
        self.assertEqual(len(response.context["data_rows"]), 1)

        # CustomFormPageSubmission have custom field. This field should appear in the listing
        self.assertContains(
            response, '<th id="useremail" class="">User email</th>', html=True
        )
        self.assertContains(response, "<td>user-m1kola@example.com</td>", html=True)

    def test_list_submissions_filtering_date_to(self):
        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"date_to": "12/31/2013"},
        )

        # Check response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailforms/index_submissions.html")
        self.assertEqual(len(response.context["data_rows"]), 1)

        # CustomFormPageSubmission have custom field. This field should appear in the listing
        self.assertContains(
            response, '<th id="useremail" class="">User email</th>', html=True
        )
        self.assertContains(response, "<td>user-john@example.com</td>", html=True)

    def test_list_submissions_filtering_range(self):
        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"date_from": "12/31/2013", "date_to": "01/02/2014"},
        )

        # Check response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailforms/index_submissions.html")
        self.assertEqual(len(response.context["data_rows"]), 1)

        # CustomFormPageSubmission have custom field. This field should appear in the listing
        self.assertContains(
            response, '<th id="useremail" class="">User email</th>', html=True
        )
        self.assertContains(response, "<td>user-m1kola@example.com</td>", html=True)

    def test_list_submissions_pagination(self):
        self.make_list_submissions()

        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"p": 2},
        )

        # Check response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailforms/index_submissions.html")

        # Check that we got the correct page
        self.assertEqual(response.context["page_obj"].number, 2)

        # CustomFormPageSubmission have custom field. This field should appear in the listing
        self.assertContains(
            response, '<th id="useremail" class="">User email</th>', html=True
        )
        self.assertContains(response, "generated-username-", count=20)

    def test_list_submissions_pagination_invalid(self):
        self.make_list_submissions()

        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"p": "Hello World!"},
        )

        # Check response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailforms/index_submissions.html")

        # Check that we got page one
        self.assertEqual(response.context["page_obj"].number, 1)

    def test_list_submissions_pagination_out_of_range(self):
        self.make_list_submissions()

        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"p": 99999},
        )

        # Check response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailforms/index_submissions.html")

        # Check that we got the last page
        self.assertEqual(
            response.context["page_obj"].number, response.context["paginator"].num_pages
        )


class TestDeleteFormSubmission(TestCase, WagtailTestUtils):
    fixtures = ["test.json"]

    def setUp(self):
        self.login(username="siteeditor", password="password")
        self.form_page = Page.objects.get(url_path="/home/contact-us/")

    def test_delete_submission_show_confirmation(self):
        response = self.client.get(
            reverse("wagtailforms:delete_submissions", args=(self.form_page.id,))
            + "?selected-submissions={}".format(FormSubmission.objects.first().id)
        )
        # Check show confirm page when HTTP method is GET
        self.assertTemplateUsed(response, "wagtailforms/confirm_delete.html")

        # Check that the deletion has not happened with GET request
        self.assertEqual(FormSubmission.objects.count(), 2)

    def test_delete_submission_with_permissions(self):
        response = self.client.post(
            reverse("wagtailforms:delete_submissions", args=(self.form_page.id,))
            + "?selected-submissions={}".format(FormSubmission.objects.first().id)
        )

        # Check that the submission is gone
        self.assertEqual(FormSubmission.objects.count(), 1)
        # Should be redirected to list of submissions
        self.assertRedirects(
            response,
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
        )

    def test_delete_multiple_submissions_with_permissions(self):
        response = self.client.post(
            reverse("wagtailforms:delete_submissions", args=(self.form_page.id,))
            + "?selected-submissions={}&selected-submissions={}".format(
                FormSubmission.objects.first().id, FormSubmission.objects.last().id
            )
        )

        # Check that both submissions are gone
        self.assertEqual(FormSubmission.objects.count(), 0)
        # Should be redirected to list of submissions
        self.assertRedirects(
            response,
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
        )

    def test_delete_submission_bad_permissions(self):
        self.login(username="eventeditor", password="password")

        response = self.client.post(
            reverse("wagtailforms:delete_submissions", args=(self.form_page.id,))
            + "?selected-submissions={}".format(FormSubmission.objects.first().id)
        )

        # Check that the user received a permission denied response
        self.assertRedirects(response, "/admin/")

        # Check that the deletion has not happened
        self.assertEqual(FormSubmission.objects.count(), 2)

    def test_delete_submission_after_filter_form_submissions_for_user_hook(self):
        # Hook forbids to delete form submissions for everyone
        def construct_forms_for_user(user, queryset):
            return queryset.none()

        with self.register_hook(
            "filter_form_submissions_for_user", construct_forms_for_user
        ):
            response = self.client.post(
                reverse("wagtailforms:delete_submissions", args=(self.form_page.id,))
                + "?selected-submissions={}".format(FormSubmission.objects.first().id)
            )

        # An user can't delete a from submission with the hook
        self.assertRedirects(response, "/admin/")
        self.assertEqual(FormSubmission.objects.count(), 2)

        # An user can delete a form submission without the hook
        response = self.client.post(
            reverse("wagtailforms:delete_submissions", args=(self.form_page.id,))
            + "?selected-submissions={}".format(
                CustomFormPageSubmission.objects.first().id
            )
        )
        self.assertEqual(FormSubmission.objects.count(), 1)
        self.assertRedirects(
            response,
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
        )


class TestDeleteCustomFormSubmission(TestCase, WagtailTestUtils):
    fixtures = ["test.json"]

    def setUp(self):
        self.login(username="siteeditor", password="password")
        self.form_page = Page.objects.get(url_path="/home/contact-us-one-more-time/")

    def test_delete_submission_show_confirmation(self):
        response = self.client.get(
            reverse("wagtailforms:delete_submissions", args=(self.form_page.id,))
            + "?selected-submissions={}".format(
                CustomFormPageSubmission.objects.first().id
            )
        )

        # Check show confirm page when HTTP method is GET
        self.assertTemplateUsed(response, "wagtailforms/confirm_delete.html")

        # Check that the deletion has not happened with GET request
        self.assertEqual(CustomFormPageSubmission.objects.count(), 2)

    def test_delete_submission_with_permissions(self):
        response = self.client.post(
            reverse("wagtailforms:delete_submissions", args=(self.form_page.id,))
            + "?selected-submissions={}".format(
                CustomFormPageSubmission.objects.first().id
            )
        )

        # Check that the submission is gone
        self.assertEqual(CustomFormPageSubmission.objects.count(), 1)
        # Should be redirected to list of submissions
        self.assertRedirects(
            response,
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
        )

    def test_delete_multiple_submissions_with_permissions(self):
        response = self.client.post(
            reverse("wagtailforms:delete_submissions", args=(self.form_page.id,))
            + "?selected-submissions={}&selected-submissions={}".format(
                CustomFormPageSubmission.objects.first().id,
                CustomFormPageSubmission.objects.last().id,
            )
        )

        # Check that both submissions are gone
        self.assertEqual(CustomFormPageSubmission.objects.count(), 0)
        # Should be redirected to list of submissions
        self.assertRedirects(
            response,
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
        )

    def test_delete_submission_bad_permissions(self):
        self.login(username="eventeditor", password="password")

        response = self.client.post(
            reverse("wagtailforms:delete_submissions", args=(self.form_page.id,))
            + "?selected-submissions={}".format(
                CustomFormPageSubmission.objects.first().id
            )
        )

        # Check that the user received a permission denied response
        self.assertRedirects(response, "/admin/")

        # Check that the deletion has not happened
        self.assertEqual(CustomFormPageSubmission.objects.count(), 2)


class TestFormsWithCustomSubmissionsList(TestCase, WagtailTestUtils):
    def create_test_user_without_admin(self, username):
        return self.create_user(username=username, password="123")

    def setUp(self):
        # Create a form page

        home_page = Page.objects.get(url_path="/home/")
        self.form_page = home_page.add_child(
            instance=FormPageWithCustomSubmissionListView(
                title="Willy Wonka Chocolate Ideas",
                slug="willy-wonka-chocolate-ideas",
                to_address="willy@wonka.com",
                from_address="info@wonka.com",
                subject="Chocolate Idea Submitted!",
            )
        )
        FormFieldForCustomListViewPage.objects.create(
            page=self.form_page,
            sort_order=1,
            label="Your email",
            field_type="email",
            required=True,
        )
        FormFieldForCustomListViewPage.objects.create(
            page=self.form_page,
            sort_order=2,
            label="Chocolate",
            field_type="singleline",
            required=True,
        )
        FormFieldForCustomListViewPage.objects.create(
            page=self.form_page,
            sort_order=3,
            label="Ingredients",
            field_type="multiline",
            required=True,
        )
        self.choices = [
            "What is chocolate?",
            "Mediocre",
            "Much excitement",
            "Wet my pants excited!",
        ]
        FormFieldForCustomListViewPage.objects.create(
            page=self.form_page,
            sort_order=4,
            label="Your Excitement",
            field_type="radio",
            required=True,
            choices=",".join(self.choices),
        )

        self.test_user_1 = self.create_test_user_without_admin("user-chocolate-maniac")
        self.test_user_2 = self.create_test_user_without_admin("user-chocolate-guy")

        # add a couple of initial form submissions for testing ordering
        new_form_submission = CustomFormPageSubmission.objects.create(
            page=self.form_page,
            user=self.test_user_1,
            form_data={
                "your_email": "new@example.com",
                "chocolate": "White Chocolate",
                "ingredients": "White colouring",
                "your_excitement": self.choices[2],
            },
        )
        if settings.USE_TZ:
            new_form_submission.submit_time = "2017-10-01T12:00:00.000Z"
        else:
            new_form_submission.submit_time = "2017-10-01T12:00:00"
        new_form_submission.save()

        old_form_submission = CustomFormPageSubmission.objects.create(
            page=self.form_page,
            user=self.test_user_2,
            form_data={
                "your_email": "old@example.com",
                "chocolate": "Dark Chocolate",
                "ingredients": "Charcoal",
                "your_excitement": self.choices[0],
            },
        )
        if settings.USE_TZ:
            old_form_submission.submit_time = "2017-01-01T12:00:00.000Z"
        else:
            old_form_submission.submit_time = "2017-01-01T12:00:00"
        old_form_submission.save()

        self.login()

    def make_list_submissions(self):
        """Make 100 submissions to test pagination on the forms submissions page"""
        for i in range(120):
            submission = CustomFormPageSubmission(
                page=self.form_page,
                user=self.test_user_1,
                form_data={
                    "your_email": "foo-%s@bar.com" % i,
                    "chocolate": "Chocolate No.%s" % i,
                    "your_excitement": self.choices[3],
                },
            )
            submission.save()

    def test_list_submissions(self):
        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,))
        )

        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailforms/index_submissions.html")
        self.assertEqual(len(response.context["data_rows"]), 2)

        # check display of list values within form submissions
        self.assertContains(response, "Much excitement")
        self.assertContains(response, "White Chocolate")
        self.assertContains(response, "Dark Chocolate")

    def test_list_submissions_pagination(self):
        self.make_list_submissions()

        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"p": 2},
        )

        # Check response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailforms/index_submissions.html")

        # test that paginate by 50 is working, should be 3 max pages (~120 values)
        self.assertContains(response, "Page 2 of 3")
        self.assertContains(response, "Wet my pants excited!", count=50)
        self.assertEqual(response.context["page_obj"].number, 2)

    def test_list_submissions_csv_export(self):
        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,)),
            {"export": "csv"},
        )

        # Check response
        self.assertEqual(response.status_code, 200)
        data_lines = response.getvalue().decode().split("\n")
        self.assertIn(
            'filename="%s-export' % self.form_page.slug,
            response.get("Content-Disposition"),
        )
        self.assertEqual(
            data_lines[0],
            "User email,Submission date,Your email,Chocolate,Ingredients,Your Excitement\r",
        )
        # first result should be the most recent as order_csv has been reversed
        if settings.USE_TZ:
            self.assertEqual(
                data_lines[1],
                "user-chocolate-maniac@example.com,2017-10-01 12:00:00+00:00,new@example.com,White Chocolate,White colouring,Much excitement\r",
            )
            self.assertEqual(
                data_lines[2],
                "user-chocolate-guy@example.com,2017-01-01 12:00:00+00:00,old@example.com,Dark Chocolate,Charcoal,What is chocolate?\r",
            )
        else:
            self.assertEqual(
                data_lines[1],
                "user-chocolate-maniac@example.com,2017-10-01 12:00:00,new@example.com,White Chocolate,White colouring,Much excitement\r",
            )
            self.assertEqual(
                data_lines[2],
                "user-chocolate-guy@example.com,2017-01-01 12:00:00,old@example.com,Dark Chocolate,Charcoal,What is chocolate?\r",
            )

    def test_list_submissions_ordering(self):
        form_submission = CustomFormPageSubmission.objects.create(
            page=self.form_page,
            user=self.create_test_user_without_admin("user-aaa-aaa"),
            form_data={
                "your_email": "new@example.com",
                "chocolate": "Old chocolate idea",
                "ingredients": "Sugar",
                "your_excitement": self.choices[2],
            },
        )
        form_submission.submit_time = "2016-01-01T12:00:00.000Z"
        form_submission.save()

        # check ordering matches default which is overridden to be 'submit_time' (oldest first)
        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,))
        )
        first_row_values = response.context["data_rows"][0]["fields"]
        self.assertIn("Old chocolate idea", first_row_values)


class TestFormsWithCustomFormBuilderSubmissionsList(TestCase, WagtailTestUtils):
    def setUp(self):
        home_page = Page.objects.get(url_path="/home/")
        form_page = home_page.add_child(
            instance=FormPageWithCustomFormBuilder(
                title="Support Request",
                slug="support-request",
                to_address="it@jenkins.com",
                from_address="support@jenkins.com",
                subject="Support Request Submitted",
            )
        )
        ExtendedFormField.objects.create(
            page=form_page,
            sort_order=1,
            label="Name",
            field_type="singleline",  # singleline field will be max_length 120
            required=True,
        )
        ExtendedFormField.objects.create(
            page=form_page,
            sort_order=1,
            label="Device IP Address",
            field_type="ipaddress",
            required=True,
        )

        for i in range(20):
            submission = FormSubmission.objects.create(
                page=form_page,
                form_data={
                    "name": "John %s" % i,
                    "device_ip_address": "192.0.2.%s" % i,
                },
            )
            submission.save()
        self.form_page = form_page
        # Login
        self.login()

    def test_list_submissions(self):
        response = self.client.get(
            reverse("wagtailforms:list_submissions", args=(self.form_page.id,))
        )

        # Check response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailforms/index_submissions.html")
        self.assertEqual(len(response.context["data_rows"]), 20)

        # check display of list values within form submissions
        self.assertContains(response, "192.0.2.1")
        self.assertContains(response, "192.0.2.15")


class TestDuplicateFormFieldLabels(TestCase, WagtailTestUtils):
    """
    If a user creates two fields with the same label, data cannot be saved correctly.
    See: https://github.com/wagtail/wagtail/issues/585
    """

    fixtures = ["test.json"]

    def setUp(self):

        self.login(username="superuser", password="password")
        # Find root page
        self.root_page = Page.objects.get(id=2)

    def test_adding_duplicate_form_labels(self):
        post_data = {
            "title": "Form page!",
            "content": "Some content",
            "slug": "contact-us",
            "form_fields-TOTAL_FORMS": "3",
            "form_fields-INITIAL_FORMS": "3",
            "form_fields-MIN_NUM_FORMS": "0",
            "form_fields-MAX_NUM_FORMS": "1000",
            "form_fields-0-id": "",
            "form_fields-0-label": "foo",
            "form_fields-0-field_type": "singleline",
            "form_fields-1-id": "",
            "form_fields-1-label": "foo",
            "form_fields-1-field_type": "singleline",
            "form_fields-2-id": "",
            "form_fields-2-label": "bar",
            "form_fields-2-field_type": "singleline",
        }
        response = self.client.post(
            reverse(
                "wagtailadmin_pages:add", args=("tests", "formpage", self.root_page.id)
            ),
            post_data,
        )

        self.assertEqual(response.status_code, 200)

        self.assertContains(
            response,
            text="There is another field with the label foo, please change one of them.",
        )

    def test_adding_duplicate_form_labels_as_cleaned_name(self):
        """
        Ensure form submission fails when attempting to create labels that will resolve
        to the same internal clean_name on the form field.
        """

        post_data = {
            "title": "Form page!",
            "content": "Some content",
            "slug": "contact-us",
            "form_fields-TOTAL_FORMS": "3",
            "form_fields-INITIAL_FORMS": "3",
            "form_fields-MIN_NUM_FORMS": "0",
            "form_fields-MAX_NUM_FORMS": "1000",
            "form_fields-0-id": "",
            "form_fields-0-label": "LOW EARTH ORBIT",
            "form_fields-0-field_type": "singleline",
            "form_fields-1-id": "",
            "form_fields-1-label": "low earth orbit",
            "form_fields-1-field_type": "singleline",
            "form_fields-2-id": "",
            "form_fields-2-label": "bar",
            "form_fields-2-field_type": "singleline",
        }
        response = self.client.post(
            reverse(
                "wagtailadmin_pages:add", args=("tests", "formpage", self.root_page.id)
            ),
            post_data,
        )

        self.assertEqual(response.status_code, 200)

        self.assertContains(
            response,
            text="There is another field with the label LOW EARTH ORBIT, please change one of them.",
        )



# -*- coding: utf-8 -*-
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailredirects", "0003_make_site_field_editable"),
    ]

    operations = [
        migrations.AlterField(
            model_name="redirect",
            name="old_path",
            field=models.CharField(
                verbose_name="Redirect from", max_length=255, db_index=True
            ),
        ),
        migrations.AlterUniqueTogether(
            name="redirect",
            unique_together={("old_path", "site")},
        ),
    ]



import re
from html import unescape

from django.db.models import Model
from django.template.loader import render_to_string
from django.utils.html import strip_tags
from django.utils.safestring import mark_safe

from wagtail.rich_text.feature_registry import FeatureRegistry
from wagtail.rich_text.rewriters import EmbedRewriter, LinkRewriter, MultiRuleRewriter

features = FeatureRegistry()


# Rewriter function to be built up on first call to expand_db_html, using the utility classes
# from wagtail.rich_text.rewriters along with the embed handlers / link handlers registered
# with the feature registry

FRONTEND_REWRITER = None


def expand_db_html(html):
    """
    Expand database-representation HTML into proper HTML usable on front-end templates
    """
    global FRONTEND_REWRITER

    if FRONTEND_REWRITER is None:
        embed_rules = features.get_embed_types()
        link_rules = features.get_link_types()
        FRONTEND_REWRITER = MultiRuleRewriter(
            [
                LinkRewriter(
                    {
                        linktype: handler.expand_db_attributes
                        for linktype, handler in link_rules.items()
                    }
                ),
                EmbedRewriter(
                    {
                        embedtype: handler.expand_db_attributes
                        for embedtype, handler in embed_rules.items()
                    }
                ),
            ]
        )

    return FRONTEND_REWRITER(html)


def get_text_for_indexing(richtext):
    """
    Return a plain text version of a rich text string, suitable for search indexing;
    like Django's strip_tags, but ensures that whitespace is left between block elements
    so that <p>hello</p><p>world</p> gives "hello world", not "helloworld".
    """
    # insert space after </p>, </h1> - </h6>, </li> and </blockquote> tags
    richtext = re.sub(
        r"(</(p|h\d|li|blockquote)>)", r"\1 ", richtext, flags=re.IGNORECASE
    )
    # also insert space after <br /> and <hr />
    richtext = re.sub(r"(<(br|hr)\s*/>)", r"\1 ", richtext, flags=re.IGNORECASE)
    return unescape(strip_tags(richtext).strip())


class RichText:
    """
    A custom object used to represent a renderable rich text value.
    Provides a 'source' property to access the original source code,
    and renders to the front-end HTML rendering.
    Used as the native value of a wagtailcore.blocks.field_block.RichTextBlock.
    """

    def __init__(self, source):
        self.source = source or ""

    def __html__(self):
        return render_to_string(
            "wagtailcore/shared/richtext.html", {"html": expand_db_html(self.source)}
        )

    def __str__(self):
        return mark_safe(self.__html__())

    def __bool__(self):
        return bool(self.source)


class EntityHandler:
    """
    An 'entity' is a placeholder tag within the saved rich text, which needs to be rewritten
    into real HTML at the point of rendering. Typically (but not necessarily) the entity will
    be a reference to a model to be fetched to have its data output into the rich text content
    (so that we aren't storing potentially changeable data within the saved rich text).

    An EntityHandler defines how this rewriting is performed.

    Currently Wagtail supports two kinds of entity: links (represented as <a linktype="...">...</a>)
    and embeds (represented as <embed embedtype="..." />).
    """

    @staticmethod
    def get_model():
        """
        If supported, returns the type of model able to be handled by this handler, e.g. Page.
        """
        raise NotImplementedError

    @classmethod
    def get_instance(cls, attrs: dict) -> Model:
        model = cls.get_model()
        return model._default_manager.get(id=attrs["id"])

    @staticmethod
    def expand_db_attributes(attrs: dict) -> str:
        """
        Given a dict of attributes from the entity tag
        stored in the database, returns the real HTML representation.
        """
        raise NotImplementedError


class LinkHandler(EntityHandler):
    pass


class EmbedHandler(EntityHandler):
    pass



import json

from django.forms import Media, widgets

from wagtail.utils.widgets import WidgetWithScript


class CustomRichTextArea(WidgetWithScript, widgets.Textarea):
    def render_js_init(self, id_, name, value):
        return "customEditorInitScript({0});".format(json.dumps(id_))

    @property
    def media(self):
        return Media(js=["vendor/custom_editor.js"])


class LegacyRichTextArea(WidgetWithScript, widgets.Textarea):
    def render_js_init(self, id_, name, value):
        return "legacyEditorInitScript({0});".format(json.dumps(id_))

    @property
    def media(self):
        return Media(js=["vendor/legacy_editor.js"])
